@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20367 $
- * $Author: csoutheren $
- * $Date: 2008-06-03 08:54:03 +0200 (mar 03 jun 2008) $
+ * $Revision: 20942 $
+ * $Author: ms30002000 $
+ * $Date: 2008-09-12 17:44:53 +0000 (Fri, 12 Sep 2008) $
  */
 
 #ifndef __OPAL_CODECPLUGIN_H
@@ -361,13 +361,17 @@ struct PluginCodec_H323GenericParameterDefinition
      zero and one (a good bet) and thus the below bit fields will be backward
      compatible, putting the parameter in all three PDU types.
    */ 
+#ifndef SOLARIS   
   struct {
+#endif  
     int collapsing:1; /* boolean */
     int excludeTCS:1;
     int excludeOLC:1;
     int excludeReqMode:1;
     int readOnly:1;
+#ifndef SOLARIS    
   };
+#endif  
 
   unsigned int id;
 
