@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 19627 $
- * $Author: csoutheren $
- * $Date: 2008-02-29 04:56:48 +0100 (ven 29 fév 2008) $
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifndef __OPAL_CHANNELS_H
@@ -38,6 +38,7 @@
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
 
 #include <rtp/rtp.h>
 #include <h323/transaddr.h>
