@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20651 $
+ * $Revision: 21004 $
  * $Author: rjongbloed $
- * $Date: 2008-08-11 04:59:27 +0200 (lun 11 aoû 2008) $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifndef __OPAL_H323EP_H
@@ -38,6 +38,7 @@
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
 
 #include <opal/rtpep.h>
 #include <opal/manager.h>
@@ -47,7 +48,7 @@
 #include <h323/h323caps.h>
 #include <h323/h235auth.h>
 
-#ifdef OPAL_H460
+#if OPAL_H460
 #include <h460/h4601.h>
 #endif
 
@@ -1322,8 +1323,7 @@ class H323EndPoint : public OpalRTPEndPoint
   //@}
 
   protected:
-    H323Gatekeeper * InternalCreateGatekeeper(H323Transport * transport);
-    PBoolean InternalRegisterGatekeeper(H323Gatekeeper * gk, PBoolean discovered);
+    bool InternalCreateGatekeeper(H323Transport * transport);
     PBoolean InternalMakeCall(
       OpalCall & call,
       const PString & existingToken,           ///<  Existing connection to be transferred
