@@ -25,6 +25,27 @@
  * The author of this code is Derek J Smithies
  *
  *  $Log: frame.h,v $
+ *  Revision 1.12  2007/09/05 04:04:35  csoutheren
+ *  Fixed spelling mistakes
+ *
+ *  Revision 1.11  2007/04/19 06:17:21  csoutheren
+ *  Fixes for precompiled headers with gcc
+ *
+ *  Revision 1.10  2007/01/23 02:08:25  dereksmithies
+ *  Handle Vnak frames correctly. handle iseqno and oseqno correctly.
+ *
+ *  Revision 1.9  2007/01/17 03:48:13  dereksmithies
+ *  Tidy up comments, remove leaks, improve reporting of packet types.
+ *
+ *  Revision 1.8  2007/01/16 03:17:42  dereksmithies
+ *  tidyup of comments. Remove unused variables.
+ *  Guarantee that media frames are sent with a monotonically increasing timestamp
+ *
+ *  Revision 1.7  2006/08/09 03:46:39  dereksmithies
+ *  Add ability to register to a remote Asterisk box. The iaxProcessor class is split
+ *  into a callProcessor and a regProcessor class.
+ *  Big thanks to Stephen Cook, (sitiveni@gmail.com) for this work.
+ *
  *  Revision 1.6  2005/08/26 03:07:38  dereksmithies
  *  Change naming convention, so all class names contain the string "IAX2"
  *
@@ -55,6 +76,20 @@
 #ifndef FRAME_H
 #define FRAME_H
 
+#ifndef _PTLIB_H
+#include <ptlib.h>
+#endif
+
+#include <opal/buildopts.h>
+
+#include <iax2/ies.h>
+#include <iax2/remote.h>
+
+#include <ptlib/sockets.h>
+
+#ifdef P_USE_PRAGMA
+#pragma interface
+#endif
 
 class IAX2Frame;
 class IAX2FrameList;
@@ -71,19 +106,11 @@ class IAX2FullFrameVideo;
 class IAX2FullFrameVoice;
 class IAX2EndPoint;
 class IAX2Processor;
+class IAX2CallProcessor;
 class IAX2IeList;
 class IAX2MiniFrame;
 class IAX2Transmitter;
 
-#include <iax2/ies.h>
-#include <iax2/remote.h>
-#include <ptlib.h>
-#include <ptlib/sockets.h>
-
-#ifdef P_USE_PRAGMA
-#pragma interface
-#endif
-
 
 
 /**Base class for holding data to be sent to a remote endpoint*/
@@ -99,19 +126,19 @@ class IAX2Frame :  public PObject
   
   /**Wait on the designated socket for an incoming UDP packet. This
      method is only called by the receiver. This method does NO interpretation*/
-  BOOL ReadNetworkPacket(PUDPSocket &sock);
+  PBoolean ReadNetworkPacket(PUDPSocket &sock);
   
   /**Interpret the data from the read process*/
-  virtual BOOL ProcessNetworkPacket();
+  virtual PBoolean ProcessNetworkPacket();
   
   /**True if this is a full frame */
-  virtual BOOL IsFullFrame();
+  virtual PBoolean IsFullFrame();
   
   /**True if it is a video frame */
-  BOOL IsVideo() const;
+  PBoolean IsVideo() const;
   
   /**True if is is an audio frame */
-  BOOL IsAudio() const;
+  PBoolean IsAudio() const;
 
   /**Pointer to the beginning of the media (after the header) in this packet.
      The low level frame has no idea on headers, so just return pointer to beginning
@@ -147,11 +174,11 @@ class IAX2Frame :  public PObject
   PINDEX   GetUnReadBytes() { return data.GetSize() - currentReadIndex; }
   
   /**Cause the header bytes for this particular frame type to be written to the internal array */
-  virtual BOOL WriteHeader() { return FALSE; }
+  virtual PBoolean WriteHeader() { return PFalse; }
   
   /**Send this packet on the specified socket to the remote host. This method is only
      called by the transmiter.*/
-  virtual BOOL TransmitPacket(PUDPSocket &sock);
+  virtual PBoolean TransmitPacket(PUDPSocket &sock);
   
   /**Pretty print this frame data to the designated stream*/
   virtual void PrintOn(ostream & strm) const;
@@ -167,15 +194,15 @@ class IAX2Frame :  public PObject
   /**Return a reference to the endpoint structure */
   IAX2EndPoint & GetEndpoint() { return endpoint; }
   
-  /**Globally unique ID string for this frame, to help track frames */
-  PString IdString() const { return PString("FR-ID#") + PString(frameIndex); }
-  
-  
+  /**Globally unique ID string for this frame, to help track frames.
+   The value returned is a pretty printed address of this frame instance.*/
+  PString IdString() const;
+    
   /**Get the timestamp as used by this class*/
   DWORD  GetTimeStamp() { return timeStamp; }
   
   /** Report flag stating that this call must be active when this frame is transmitted*/
-  virtual BOOL CallMustBeActive() { return TRUE; }     
+  virtual PBoolean CallMustBeActive() { return PTrue; }     
   
   /**Specify the type of this frame. */
   enum IAX2FrameType {
@@ -202,14 +229,16 @@ class IAX2Frame :  public PObject
    Whenever a frame is transmitted, this method will be called.*/
   virtual void InitialiseHeader(IAX2Processor * /*processor*/) { }
   
-  /**Return true if this frame should be retransmitted. Acks are never retransmitted. cmdNew are retransmitted.*/
-  BOOL CanRetransmitFrame() const {return canRetransmitFrame; } 
+  /**Return true if this frame should be retransmitted. Acks are never
+     retransmitted. cmdNew are retransmitted.*/
+  PBoolean CanRetransmitFrame() const {return canRetransmitFrame; } 
   
-  /**Get the string which uniquely identifies the IAXConnection that sent this frame */
+  /**Get the string which uniquely identifies the IAXConnection that
+     sent this frame */
   PString GetConnectionToken() const { return connectionToken; }
 
-  /**Set the string which uniquely identifies the IAXConnection that is responsible for
-     this frame */
+  /**Set the string which uniquely identifies the IAXConnection that
+     is responsible for this frame */
   void SetConnectionToken(PString newToken) { connectionToken = newToken; } 
 
   /**Create the connection token id, which uniquely identifies the
@@ -218,7 +247,7 @@ class IAX2Frame :  public PObject
 
   /**Write the data in the variables to this frame's data array. If
      encryption is on, the data will be encrypted */
-  BOOL EncryptContents(IAX2Encryption &encData);
+  PBoolean EncryptContents(IAX2Encryption &encData);
 
   /**Get the offset to the beginning of the encrypted region */
   virtual PINDEX GetEncryptionOffset();
@@ -227,37 +256,47 @@ class IAX2Frame :  public PObject
 
   /**Use the supplied encryptionKey, and data in storage, to decrypt this frame.
    
-  Return False if the decryption fails, TRUE if the decryption works.*/
-  BOOL DecryptContents(IAX2Encryption & encryption);
+  Return False if the decryption fails, PTrue if the decryption works.*/
+  PBoolean DecryptContents(IAX2Encryption & encryption);
 
-  /**Specification of the location (address, call number etc) of the far endpoint */
+  /**Specification of the location (address, call number etc) of the
+     far endpoint */
   IAX2Remote  remote;
   
-  /**Variable specifying the IAX type of frame that this is. Used only in reading from the network */
+  /**Variable specifying the IAX type of frame that this is. Used only
+     in reading from the network */
   IAX2FrameType frameType;
   
-  /** Read 1 byte from the internal area, (Internal area is filled when reading the packet in). Big Endian.*/
-  BOOL          Read1Byte(BYTE & res);
+  /** Read 1 byte from the internal area, (Internal area is filled
+      when reading the packet in). Big Endian.*/
+  PBoolean          Read1Byte(BYTE & res);
   
-  /** Read 2 bytes from the internal area, (Internal area is filled when reading the packet in)  Big Endian.*/
-  BOOL          Read2Bytes(PINDEX & res);
+  /** Read 2 bytes from the internal area, (Internal area is filled
+      when reading the packet in) Big Endian.*/
+  PBoolean          Read2Bytes(PINDEX & res);
   
-  /** Read 2 bytes from the internal area, (Internal area is filled when reading the packet in)  Big Endian.*/
-  BOOL          Read2Bytes(WORD & res);
+  /** Read 2 bytes from the internal area, (Internal area is filled
+      when reading the packet in) Big Endian.*/
+  PBoolean          Read2Bytes(WORD & res);
   
-  /** Read 4 bytes from the internal area, (Internal area is filled when reading the packet in)  Big Endian.*/
-  BOOL          Read4Bytes(DWORD & res);
+  /** Read 4 bytes from the internal area, (Internal area is filled
+      when reading the packet in) Big Endian.*/
+  PBoolean          Read4Bytes(DWORD & res);
   
-  /** Write 1 byte to the internal area, as part of writing the header info */
+  /** Write 1 byte to the internal area, as part of writing the header
+      info */
   void          Write1Byte(BYTE newVal);
   
-  /** Write 1 byte to the internal area, as part of writing the header info. Send only the lowest 8 bits of source*/
+  /** Write 1 byte to the internal area, as part of writing the header
+      info. Send only the lowest 8 bits of source*/
   void          Write1Byte(PINDEX newVal);
   
-  /** Write 2 bytes to the internal area, as part of writing the header info  Big Endian.*/
+  /** Write 2 bytes to the internal area, as part of writing the
+      header info Big Endian.*/
   void          Write2Bytes(PINDEX newVal);
   
-  /** Write 4 bytes to the internal area, as part of writing the header info  Big Endian.*/
+  /** Write 4 bytes to the internal area, as part of writing the
+      header info Big Endian.*/
   void          Write4Bytes(unsigned int newVal);
   
   /** Initialise all internal storage in this structrue */
@@ -266,17 +305,18 @@ class IAX2Frame :  public PObject
   /**Reference to the global variable of this program */
   IAX2EndPoint      & endpoint;
   
-  /**Internal storage array, ready for sending to remote node, or ready for receiving from remote node*/
+  /**Internal storage array, ready for sending to remote node, or
+     ready for receiving from remote node*/
   PBYTEArray         data;
   
   /**Flag to indicate if this is a MiniFrame or FullFrame */
-  BOOL               isFullFrame;
+  PBoolean               isFullFrame;
   
   /**Flag to indicate if this is a MiniFrame with video */
-  BOOL               isVideo;
+  PBoolean               isVideo;
   
   /**Flag to indicate if this is a MiniFrame with audio */
-  BOOL               isAudio;
+  PBoolean               isAudio;
   
   /**Index of where we are reading from the internal data area */
   PINDEX               currentReadIndex;  
@@ -287,11 +327,8 @@ class IAX2Frame :  public PObject
   /**unsigned 32 bit representaiton of the time of this day */
   DWORD                timeStamp;  
   
-  /** Internal variable that uniquely identifies this frame */
-  int                frameIndex;
-  
   /**Indicate if this frame can be retransmitted*/
-  BOOL               canRetransmitFrame;
+  PBoolean               canRetransmitFrame;
 
   /**Connection Token, which uniquely identifies the IAXConnection
      that sent this frame. The token will (except for the first setup
@@ -299,9 +336,9 @@ class IAX2Frame :  public PObject
      this incoming frame.  */
   PString            connectionToken;
 
-  /**The time stamp to use, for those cases when the user demands a particular
-   * timestamp on construction. */
-  PINDEX presetTimeStamp;
+  /**The time stamp to use, for those cases when the user demands a
+   * particular timestamp on construction. */
+  DWORD presetTimeStamp;
 };
 
 /////////////////////////////////////////////////////////////////////////////    
@@ -312,7 +349,7 @@ class IAX2MiniFrame : public IAX2Frame
  public:
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2MiniFrame(IAX2Frame & srcFrame);
+  IAX2MiniFrame(const IAX2Frame & srcFrame);
   
   /** Construction from an endpoint, to create an empty frame. */
   IAX2MiniFrame(IAX2EndPoint & _endpoint); 
@@ -321,24 +358,27 @@ class IAX2MiniFrame : public IAX2Frame
      PBYTEArray), in preparation to sending to remote node.  The
      constructor will not delete the supplied PBYTEArray structure.
 
-     TimeStamp will be calculated from time since call started, if the users timestamp is 0.
-     If the users timeStamp is non zero, the frames timestamp will be this.
+     TimeStamp will be calculated from time since call started, if the
+     users timestamp is 0.  If the users timeStamp is non zero, the
+     frames timestamp will be this.
   */
-  IAX2MiniFrame(IAX2Processor * con, PBYTEArray &sound, BOOL isAudio, PINDEX usersTimeStamp = 0);
+  IAX2MiniFrame(IAX2Processor * con, PBYTEArray &sound, PBoolean isAudio, DWORD usersTimeStamp = 0);
 
   /**Destructor*/
   virtual ~IAX2MiniFrame();
   
-  /** Process the incoming frame some more, but process it as this frame type demands*/
-  virtual BOOL ProcessNetworkPacket();
+  /** Process the incoming frame some more, but process it as this
+      frame type demands*/
+  virtual PBoolean ProcessNetworkPacket();
   
   /**Write the header to the internal data area */
-  virtual BOOL WriteHeader();
+  virtual PBoolean WriteHeader();
   
   /**Pretty print this frame data to the designated stream*/
   virtual void PrintOn(ostream & strm) const;
   
-  /**Pointer to the beginning of the media (after the header) in this packet */
+  /**Pointer to the beginning of the media (after the header) in this
+     packet */
   virtual BYTE *GetMediaDataPointer();
   
   /**Number of bytes in the media section of this packet. */
@@ -362,6 +402,7 @@ class IAX2MiniFrame : public IAX2Frame
   void ZeroAllValues();
 };
 
+/////////////////////////////////////////////////////////////////////////////    
 /////////////////////////////////////////////////////////////////////////////    
 /**Class to handle a full frame, which is sent reliably to the remote endpoint */
 class IAX2FullFrame : public IAX2Frame
@@ -370,7 +411,7 @@ class IAX2FullFrame : public IAX2Frame
  public:
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrame(IAX2Frame & srcFrame);
+  IAX2FullFrame(const IAX2Frame & srcFrame);
   
   /** Construction from an endpoint, to create an empty frame. 
       In this case, the class is filled with the various methods*/
@@ -381,44 +422,67 @@ class IAX2FullFrame : public IAX2Frame
   virtual ~IAX2FullFrame();
   
   /**Return True if this an ack frame */
-  BOOL IsAckFrame() { return isAckFrame; }
+  PBoolean IsAckFrame() { return isAckFrame; }
   
   /**Return True if this is a PING frame */
-  BOOL IsPingFrame();
+  PBoolean IsPingFrame();
 
   /**Return True if this is a NEW frame */
-  BOOL IsNewFrame();
+  PBoolean IsNewFrame();
 
   /**Return True if this is a LAGRQ frame */
-  BOOL IsLagRqFrame();
+  PBoolean IsLagRqFrame();
 
   /**Return True if this is a LAGRP frame */
-  BOOL IsLagRpFrame();
+  PBoolean IsLagRpFrame();
 
   /**Return True if this is a PONG frame */
-  BOOL IsPongFrame();
+  PBoolean IsPongFrame();
 
   /**Return True if this is a AuthReq frame */
-  BOOL IsAuthReqFrame();
+  PBoolean IsAuthReqFrame();
 
   /**Return True if this is a VNAK frame */
-  BOOL IsVNakFrame();
-
-  /**Return True if this FullFrame is of a type that increments the InSeqNo */
-  BOOL FrameIncrementsInSeqNo();
-
-  /**True if this is a full frame - always returns true as this is a full frame. */
-  virtual BOOL IsFullFrame() { return TRUE; }  
+  PBoolean IsVnakFrame();
   
+  /**Return True if this is a REGREQ frame */
+  PBoolean IsRegReqFrame();
+  
+  /**Return True if this is a REGAUTH frame */
+  PBoolean IsRegAuthFrame();
+  
+  /**Return True if this is a REGACK frame */
+  PBoolean IsRegAckFrame();  
+  
+  /**Return True if this is a REGREL frame */
+  PBoolean IsRegRelFrame();
+  
+  /**Return True if this is a REGREJ frame */
+  PBoolean IsRegRejFrame();
+
+  /**Return True if this FullFrame is of a type that increments the
+     InSeqNo */
+  PBoolean FrameIncrementsInSeqNo();
+
+  /**True if this is a full frame - always returns true as this is a
+     full frame. */
+  virtual PBoolean IsFullFrame() { return PTrue; }  
+  
+  /**Report PTrue if this is a hangup frame. We need this information
+     for processing incoming frames, before fully dissection of the
+     frame has completed */
+  PBoolean IsHangupFrame();
+
   /** Initialise to zero all the members of this particular class */
   void ZeroAllValues();
   
-  /** Process the incoming frame some more, but process it as a full frame */
-  virtual BOOL ProcessNetworkPacket();
+  /** Process the incoming frame some more, but process it as a full
+      frame */
+  virtual PBoolean ProcessNetworkPacket();
   
-  /**Send this packet on the specified socket to the remote host. This method is only
-     called by the transmiter.*/
-  virtual BOOL TransmitPacket(PUDPSocket &sock);
+  /**Send this packet on the specified socket to the remote host. This
+     method is only called by the transmiter.*/
+  virtual PBoolean TransmitPacket(PUDPSocket &sock);
   
   /**Get text descrption of this frame type*/
   PString GetFullFrameName() const;
@@ -427,28 +491,37 @@ class IAX2FullFrame : public IAX2Frame
   virtual PString GetSubClassName() const
     { return PString(" subclass=") + PString(subClass); }
   
-  /**Stop the timer, so this packet is not retransmitted. Mark packet as dead. This happens
-     when a packet has been received that matches one of the previously sent packets. */
+  /**Stop the timer, so this packet is not retransmitted. Mark packet
+     as dead. This happens when a packet has been received that
+     matches one of the previously sent packets. */
   void MarkDeleteNow();
   
-  /**Pointer to the beginning of the media (after the header) in this packet */
+  /**A Vnak frame has been received. This Vnak frame is demanding that
+     we resend this particular frame. Given it is to be resent by
+     vnak, we reset the countdown variables. E.G. it gets the full
+     amount of retries again. */
+  void MarkVnakSendNow();
+
+  /**Pointer to the beginning of the media (after the header) in this
+     packet */
   virtual BYTE *GetMediaDataPointer();
   
   /**Number of bytes in the media section of this packet. */
   virtual PINDEX GetMediaDataSize();
   
-  /**Deteremine the current value of the subClass variable */
+  /**Determine the current value of the subClass variable */
   PINDEX GetSubClass() const { return subClass; }
   
   /**Dry the current value of the subClass variable */
-  void SetSubClass(int newValue) { subClass = newValue;}
+  void SetSubClass(PINDEX newValue) { subClass = newValue;}
   
   /**Write the header for this class to the internal data array. 12
-     bytes of data are writen.  The application developer must write the
-     remaining bytes, before transmiting this frame. */
-  virtual BOOL WriteHeader();
+     bytes of data are writen.  The application developer must write
+     the remaining bytes, before transmiting this frame. */
+  virtual PBoolean WriteHeader();
   
-  /**Alter the two bytes for in and out sequence values. (in the header)*/
+  /**Alter the two bytes for in and out sequence values. (in the
+     header)*/
   void ModifyFrameHeaderSequenceNumbers(PINDEX inNo, PINDEX outNo);
 
   /**Alter the four bytes for this frames timestamp. It is required,
@@ -458,13 +531,15 @@ class IAX2FullFrame : public IAX2Frame
   void ModifyFrameTimeStamp(PINDEX newTimeStamp);
 
   /**Mark this frame as having (or not having) information elements*/
-  virtual BOOL InformationElementsPresent() { return FALSE; }  
+  virtual PBoolean InformationElementsPresent() { return PFalse; }  
   
-  /**Get flag to see if this frame is ready to be sent (or resent). In other words, has the timer expired?*/
-  BOOL  SendFrameNow() { return sendFrameNow; }
+  /**Get flag to see if this frame is ready to be sent (or resent). In
+     other words, has the timer expired?*/
+  PBoolean  SendFrameNow() { return sendFrameNow; }
   
-  /**Get flag to see if this frame is ready for deletion. In other words. Has it been sent too many times? */
-  BOOL  DeleteFrameNow() { return deleteFrameNow; }
+  /**Get flag to see if this frame is ready for deletion. In other
+     words. Has it been sent too many times? */
+  PBoolean  DeleteFrameNow() { return deleteFrameNow; }
   
   /**Get the sequence number info (inSeqNo and outSeqNo) */
   IAX2SequenceNumbers & GetSequenceInfo() { return sequence; }
@@ -477,7 +552,7 @@ class IAX2FullFrame : public IAX2Frame
   
   /**Compare this FullFrame with another full frame, which is used when determining if we are
      dealing with a frame we have already processed */
-  BOOL operator *= (IAX2FullFrame & other);
+  PBoolean operator *= (IAX2FullFrame & other);
   
   /**enum to define if the call must be active when sending this
      frame*/
@@ -494,7 +569,7 @@ class IAX2FullFrame : public IAX2Frame
   
  protected:
   /** Report flag stating that this call must be active when this frame is transmitted*/
-  virtual BOOL CallMustBeActive() { return callMustBeActive; }
+  virtual PBoolean CallMustBeActive() { return callMustBeActive; }
   
   /**Turn the 8 bit subClass value into a 16 bit representation */
   void UnCompressSubClass(BYTE a);
@@ -539,26 +614,26 @@ class IAX2FullFrame : public IAX2Frame
   enum RetryTime {
     minRetryTime = 500,    /*!< 500 milliseconds     */
     maxRetryTime = 010000, /*!< 10 seconds           */
-    maxRetries   = 10,     /*!< number of retries    */
+    maxRetries   = 3       /*!< number of retries    */
   };
   
   /**Class holding the sequence numbers, which is used by all classes which have a FullFrame ancestor. */
   IAX2SequenceNumbers sequence;
   
   /**List flag, indicating if this frame ready for sending*/
-  BOOL         sendFrameNow;   
+  PBoolean         sendFrameNow;   
   
   /**List flag, this frame is ready for deletion (too many retries)*/
-  BOOL         deleteFrameNow; 
+  PBoolean         deleteFrameNow; 
   
   /**A tracking flag to indicate this fame has been resent*/
-  BOOL         packetResent;   
+  PBoolean         packetResent;   
   
   /** Flag stating that this call must be active when this frame is transmitted  */
-  BOOL callMustBeActive;
+  PBoolean callMustBeActive;
   
   /** flag to indicate if this is an ack frame */
-  BOOL isAckFrame;  
+  PBoolean isAckFrame;  
 };
 
 /////////////////////////////////////////////////////////////////////////////    
@@ -573,11 +648,11 @@ class IAX2FullFrameDtmf : public IAX2FullFrame
  public:
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameDtmf(IAX2Frame & srcFrame);
+  IAX2FullFrameDtmf(const IAX2Frame & srcFrame);
 
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameDtmf(IAX2FullFrame & srcFrame);
+  IAX2FullFrameDtmf(const IAX2FullFrame & srcFrame);
   
   /**Construction from a Connection class. 
      Classes generated from this are then on sent to a remote endpoint. */
@@ -636,11 +711,11 @@ class IAX2FullFrameVoice : public IAX2FullFrame
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameVoice(IAX2Frame & srcFrame);
+  IAX2FullFrameVoice(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameVoice(IAX2FullFrame & srcFrame);
+  IAX2FullFrameVoice(const IAX2FullFrame & srcFrame);
   
   /**Construction from an encoded audio array (stored in a
      PBYTEArray), in preparation to sending to remote node.  The
@@ -650,7 +725,7 @@ class IAX2FullFrameVoice : public IAX2FullFrame
      specified timeStamp == 0, the timeStamp will be calculated from when the
      call started.
   */
-  IAX2FullFrameVoice(IAX2Processor *processor, PBYTEArray &sound, 
+  IAX2FullFrameVoice(IAX2CallProcessor *processor, PBYTEArray &sound, 
 		 PINDEX usersTimeStamp = 0);
   
   /**Declare an empty destructor */
@@ -713,11 +788,11 @@ class IAX2FullFrameVideo : public IAX2FullFrame
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameVideo(IAX2Frame & srcFrame);
+  IAX2FullFrameVideo(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameVideo(IAX2FullFrame & srcFrame);
+  IAX2FullFrameVideo(const IAX2FullFrame & srcFrame);
   
   /**Get text description of the subclass contents*/
   virtual PString GetSubClassName() const;
@@ -727,7 +802,7 @@ class IAX2FullFrameVideo : public IAX2FullFrame
     jpeg  = 0x10000,   /*!< Jpeg video format in this frame     */
     png   = 0x20000,   /*!< PNG video format in this frame      */
     h261  = 0x40000,   /*!< H.261 video format in this frame    */
-    h263  = 0x80000    /*!< H.263 video fromat in this frame    */
+    h263  = 0x80000    /*!< H.263 video format in this frame    */
   };
   
   /**Return the FullFrame type represented here (voice, protocol, session etc*/
@@ -772,11 +847,11 @@ class IAX2FullFrameSessionControl : public IAX2FullFrame
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameSessionControl(IAX2Frame & srcFrame);
+  IAX2FullFrameSessionControl(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameSessionControl(IAX2FullFrame & srcFrame);
+  IAX2FullFrameSessionControl(const IAX2FullFrame & srcFrame);
   
   /**Construction from a Connection class. 
      Classes generated from this are then on sent to a remote endpoint. */
@@ -820,12 +895,12 @@ class IAX2FullFrameNull : public IAX2FullFrame
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet.
      Classes generated like this are used to handle received data. */
-  IAX2FullFrameNull(IAX2Frame & srcFrame);
+  IAX2FullFrameNull(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet.
      Classes generated like this are used to handle received data. */
-  IAX2FullFrameNull(IAX2FullFrame & srcFrame);
+  IAX2FullFrameNull(const IAX2FullFrame & srcFrame);
   
   /**Get text description of the subclass contents*/
   virtual PString GetSubClassName() const { return  PString(""); }
@@ -862,8 +937,8 @@ class IAX2FullFrameProtocol : public IAX2FullFrame
     cmdInval     =  10,      /*!< Destroy this call immediatly    */
     cmdLagRq     =  11,      /*!< Initial message, used to measure the round trip time    */
     cmdLagRp     =  12,      /*!< Reply to cmdLagrq, which tells us the round trip time    */
-    cmdRegReq    =  13,      /*!< Request fore Registration    */
-    cmdRegAuth   =  14,      /*!< Registration is required for  authentication    */
+    cmdRegReq    =  13,      /*!< Request for Registration    */
+    cmdRegAuth   =  14,      /*!< Registration requires for authentication    */
     cmdRegAck    =  15,      /*!< Registration has been accepted    */
     cmdRegRej    =  16,      /*!< Registration has been rejected    */
     cmdRegRel    =  17,      /*!< Force the release of the current registration    */
@@ -892,12 +967,12 @@ class IAX2FullFrameProtocol : public IAX2FullFrame
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet.
      Classes generated like this are used to handle received data. */
-  IAX2FullFrameProtocol(IAX2Frame & srcFrame);
+  IAX2FullFrameProtocol(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet.
      Classes generated like this are used to handle received data. */
-  IAX2FullFrameProtocol(IAX2FullFrame & srcFrame);
+  IAX2FullFrameProtocol(const IAX2FullFrame & srcFrame);
   
   /**Construction from a Connection class. 
      Classes generated from this are then on sent to a remote endpoint. */
@@ -934,10 +1009,16 @@ class IAX2FullFrameProtocol : public IAX2FullFrame
   void SetRetransmissionRequired();
   
   /**Mark this frame as having (or not having) information elements*/
-  virtual BOOL InformationElementsPresent() { return !ieElements.IsEmpty(); }
+  virtual PBoolean InformationElementsPresent() { return !ieElements.IsEmpty(); }
   
+  /**Report the current value of the subClass variable */
+  ProtocolSc GetSubClass() const { return (ProtocolSc) subClass; }
+
   /**Get text description of the subclass contents*/
   virtual PString GetSubClassName() const; 
+
+  /**Get text description of the subclass contents*/
+  static PString GetSubClassName(PINDEX t);
   
   /**Return a pointer to the n'th Ie in the internal list. If it is not
      there, a NULL is returned */
@@ -962,14 +1043,22 @@ class IAX2FullFrameProtocol : public IAX2FullFrame
   /**Return the IAX2FullFrame type represented here (voice, protocol, session etc*/
   virtual BYTE GetFullFrameType() { return iax2ProtocolType; }
   
+  /**Pretty print this frame data to the designated stream*/
+  virtual void PrintOn(ostream & strm) const;
+
  protected:
   
   /**Read the information elements from the incoming data array 
      to generate a list of information element classes*/
-  BOOL ReadInformationElements();
+  PBoolean ReadInformationElements();
   
   /**A list of the IEs read from/(or  written to) the data section of this frame,*/
   IAX2IeList ieElements;
+
+  /**Pretty print the protocol value with an English word. */
+#if PTRACING
+    friend ostream & operator<<(ostream & o, ProtocolSc t);
+#endif
 };
 
 /////////////////////////////////////////////////////////////////////////////    
@@ -991,11 +1080,11 @@ class IAX2FullFrameText : public IAX2FullFrame
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameText(IAX2Frame & srcFrame);
+  IAX2FullFrameText(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameText(IAX2FullFrame & srcFrame);
+  IAX2FullFrameText(const IAX2FullFrame & srcFrame);
   
   /**Get text description of the subclass contents*/
   virtual PString GetSubClassName() const;
@@ -1024,11 +1113,11 @@ class IAX2FullFrameImage : public IAX2FullFrame
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameImage(IAX2Frame & srcFrame);
+  IAX2FullFrameImage(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameImage(IAX2FullFrame & srcFrame);
+  IAX2FullFrameImage(const IAX2FullFrame & srcFrame);
   
   /**Get text description of the subclass contents*/
   virtual PString GetSubClassName() const;
@@ -1051,11 +1140,11 @@ class IAX2FullFrameHtml : public IAX2FullFrame
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameHtml(IAX2Frame & srcFrame);
+  IAX2FullFrameHtml(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameHtml(IAX2FullFrame & srcFrame);
+  IAX2FullFrameHtml(const IAX2FullFrame & srcFrame);
   
   /**Get text description of the subclass contents*/
   virtual PString GetSubClassName() const;
@@ -1077,11 +1166,11 @@ class IAX2FullFrameCng : public IAX2FullFrame
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameCng(IAX2Frame & srcFrame);
+  IAX2FullFrameCng(const IAX2Frame & srcFrame);
   
   /**Construction from a supplied dataframe.
      In this case, this class is filled from an incoming data packet*/
-  IAX2FullFrameCng(IAX2FullFrame & srcFrame);
+  IAX2FullFrameCng(const IAX2FullFrame & srcFrame);
   
   /**Get text description of the subclass contents*/
   virtual PString GetSubClassName() const;
@@ -1108,24 +1197,29 @@ class IAX2FrameList : public IAX2Frame *
   ~IAX2FrameList();
   
   /**Report the frames queued in this list*/
-  void ReportList();
+  void ReportList(PString & answer);
   
   /**Get pointer to last frame in the list. Remove this frame from the list */
   IAX2Frame *GetLastFrame();
   
   /**Removing item from list will not automatically delete it */
-  void Initialise() {  DisallowDeleteObjects(); }
+  void Initialise();
     
   /**True if this frame list is empty*/
-  BOOL Empty() { return GetSize() == 0; }
+  PBoolean Empty() { return GetSize() == 0; }
   
   /**Copy to this frame the contents of the frameList pointed to by src*/
   void GrabContents(IAX2FrameList &src);
   
   /**Delete the frame that has been sent, which is waiting for this
-   * reply. The reply is the argument. */
+     reply. The reply is the argument. */
   void DeleteMatchingSendFrame(IAX2FullFrame *reply);
-  
+
+  /** A Vnak frame has been received (voice not acknowledged) which actually
+      means, retransmit all those frames you have on this particular call
+      number from the oseqno specified in the supplied frame */
+  void SendVnakRequestedFrames(IAX2FullFrameProtocol &src);
+
   /**Add the frame (supplied as an argument) to the end of this list*/
   void AddNewFrame(IAX2Frame *src);
   
@@ -1146,18 +1240,27 @@ class IAX2FrameList : public IAX2Frame *
   PMutex mutex;
 };
 
-
+/////////////////////////////////////////////////////////////////////////////
+/**The class IAX2ActiveFrameList is essentially the same as
+   IAX2FrameList, except that it is initialised (by default) and the
+   user is not required to use the Initialise method. This class will
+   not ever automatically delete members when they are removed */
+class IAX2ActiveFrameList : public IAX2FrameList
+{
+  PCLASSINFO(IAX2ActiveFrameList, IAX2FrameList);
+ public:
+  IAX2ActiveFrameList() { Initialise(); }
+};
 /////////////////////////////////////////////////////////////////////////////    
 
 
 #endif // FRAME_H
 /* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
+/* With the comment below, the tab key does auto indent to 2 spaces.     */
 
 /*
  * Local Variables:
  * mode:c
- * c-file-style:linux
  * c-basic-offset:2
  * End:
  */
