@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20551 $
+ * $Revision: 21008 $
  * $Author: rjongbloed $
- * $Date: 2008-07-08 09:19:07 +0200 (mar 08 jui 2008) $
+ * $Date: 2008-09-16 10:13:12 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifndef __OPAL_LID_H
@@ -38,6 +38,7 @@
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
 
 #include <opal/mediafmt.h>
 
@@ -588,6 +589,7 @@ class OpalLineInterfaceDevice : public PObject
       CongestionTone,// aka fast busy tone
       ClearTone,   // Call failed/disconnected tone (often same as busy tone)
       MwiTone,     // Message Waiting Tone
+      RoutingTone, // Call is being routed (not normal for PSTN, but VoIP can take a while)
       CNGTone,     // Fax CNG tone
       CEDTone,     // Fax CED tone
       NumTones
@@ -987,7 +989,7 @@ class OpalLine : public PObject
       */
     virtual PBoolean IsRinging(
       DWORD * cadence = NULL  ///<  Cadence of incoming ring
-    ) { return device.IsLineRinging(lineNumber, cadence); }
+    );
 
     /**Get the number of rings.
        If the line is ringing then 
