@@ -24,92 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: call.h,v $
- * Revision 2.24  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.23  2005/10/22 12:16:05  dsandras
- * Moved mutex preventing media streams to be opened before they are completely closed to the SIPConnection class.
- *
- * Revision 2.22  2005/09/22 17:08:52  dsandras
- * Added mutex to protect media streams access and prevent media streams for a call to be closed before they are all opened.
- *
- * Revision 2.21  2005/08/04 17:21:48  dsandras
- * Added functions to close/remove the media streams of a call.
- *
- * Revision 2.20  2005/07/11 01:52:23  csoutheren
- * Extended AnsweringCall to work for SIP as well as H.323
- * Fixed problems with external RTP connection in H.323
- * Added call to OnClosedMediaStream
- *
- * Revision 2.19  2005/05/11 04:25:09  dereksmithies
- * Add description of the OpalConnection class instances managed by an OpalCall structure.
- *
- * Revision 2.18  2004/08/14 07:56:29  rjongbloed
- * Major revision to utilise the PSafeCollection classes for the connections and calls.
- *
- * Revision 2.17  2004/07/14 13:26:14  rjongbloed
- * Fixed issues with the propagation of the "established" phase of a call. Now
- *   calling an OnEstablished() chain like OnAlerting() and OnConnected() to
- *   finally arrive at OnEstablishedCall() on OpalManager
- *
- * Revision 2.16  2004/05/01 10:00:50  rjongbloed
- * Fixed ClearCallSynchronous so now is actually signalled when call is destroyed.
- *
- * Revision 2.15  2004/04/18 07:09:12  rjongbloed
- * Added a couple more API functions to bring OPAL into line with similar functions in OpenH323.
- *
- * Revision 2.14  2004/03/11 06:54:27  csoutheren
- * Added ability to disable SIP or H.323 stacks
- *
- * Revision 2.13  2004/02/07 00:35:46  rjongbloed
- * Fixed calls GetMediaFormats so no DOES return intersection of all connections formats.
- * Tidied some API elements to make usage more explicit.
- *
- * Revision 2.12  2003/06/02 03:13:28  rjongbloed
- * Made changes so that media stream in opposite direction to the one already
- *   opened will use same media format for preference. That is try and use
- *   symmetric codecs if possible.
- *
- * Revision 2.11  2003/03/06 03:57:46  robertj
- * IVR support (work in progress) requiring large changes everywhere.
- *
- * Revision 2.10  2002/11/10 11:33:17  robertj
- * Updated to OpenH323 v1.10.3
- *
- * Revision 2.9  2002/09/16 02:52:35  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 2.8  2002/04/08 02:40:13  robertj
- * Fixed issues with using double originate call, eg from simple app command line.
- *
- * Revision 2.7  2002/02/19 07:42:25  robertj
- * Restructured media bypass functions to fix problems with RFC2833.
- *
- * Revision 2.6  2002/02/11 07:38:01  robertj
- * Added media bypass for streams between compatible protocols.
- *
- * Revision 2.5  2002/01/22 05:03:47  robertj
- * Revamp of user input API triggered by RFC2833 support
- *
- * Revision 2.4  2001/11/15 07:02:12  robertj
- * Changed OpalCall::OpenSourceMediaStreams so the connection to not open
- *   a media stream on is optional.
- *
- * Revision 2.3  2001/08/22 10:20:09  robertj
- * Changed connection locking to use double mutex to guarantee that
- *   no threads can ever deadlock or access deleted connection.
- *
- * Revision 2.2  2001/08/17 08:24:33  robertj
- * Added call end reason for whole call, not just connection.
- *
- * Revision 2.1  2001/08/01 05:28:59  robertj
- * Added function to get all media formats possible in a call.
- *
- * Revision 2.0  2001/07/27 15:48:24  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
+ * $Revision: 20801 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-01 03:05:12 +0000 (Mon, 01 Sep 2008) $
  */
 
 #ifndef __OPAL_CALL_H
@@ -176,10 +93,13 @@ class OpalCall : public PSafeObject
   //@{
     /**Indicate tha all connections in call are connected and media is going.
       */
-    BOOL IsEstablished() const { return isEstablished; }
+    PBoolean IsEstablished() const { return isEstablished; }
 
     /**Call back to indicate that the call has been established.
-       At this point in time there are no connections left in the call.
+       At this point in time every connection in the call is in the
+       "Established" state. This is a better function than using
+       OnEstablished() on a particular connection as it assures that
+       bi-direction media is flowing.
 
        The default behaviour is to call OpalManager::OnEstablishedCall().
       */
@@ -226,12 +146,21 @@ class OpalCall : public PSafeObject
 
   /**@name Connection management */
   //@{
+    /**Call back for a new connection has been constructed.
+       This is called after CreateConnection has returned a new connection.
+       It allows an application to make any custom adjustments to the
+       connection before it begins to process the protocol. behind it.
+      */
+    virtual void OnNewConnection(
+      OpalConnection & connection   ///< New connection just created
+    );
+
     /**Call back for SetUp conenction.
 
        The default behaviour is to call SetUpConnection() on all the other
        connections in the call.
       */
-    virtual BOOL OnSetUp(
+    virtual PBoolean OnSetUp(
       OpalConnection & connection   ///<  Connection that indicates it is alerting
     );
 
@@ -240,17 +169,29 @@ class OpalCall : public PSafeObject
        The default behaviour is to call SetAlerting() on all the other
        connections in the call.
       */
-    virtual BOOL OnAlerting(
+    virtual PBoolean OnAlerting(
       OpalConnection & connection   ///<  Connection that indicates it is alerting
     );
 
-    virtual OpalConnection::AnswerCallResponse
-       OnAnswerCall(OpalConnection & connection,
-                     const PString & caller
-    );
+    /**Call back for answering an incoming call.
+       This function is called after the connection has been acknowledged
+       but before the connection is established
 
-    virtual void AnsweringCall(
-      OpalConnection::AnswerCallResponse response ///<  Answer response to incoming call
+       This gives the application time to wait for some event before
+       signalling to the endpoint that the connection is to proceed. For
+       example the user pressing an "Answer call" button.
+
+       If AnswerCallDenied is returned the connection is aborted and the
+       connetion specific end call PDU is sent. If AnswerCallNow is returned 
+       then the connection proceeding, Finally if AnswerCallPending is returned then the
+       protocol negotiations are paused until the AnsweringCall() function is
+       called.
+
+       The default behaviour returns AnswerCallPending.
+     */
+    virtual OpalConnection::AnswerCallResponse OnAnswerCall(
+      OpalConnection & connection,
+      const PString & caller
     );
 
     /**A call back function whenever a connection is "connected".
@@ -264,13 +205,14 @@ class OpalCall : public PSafeObject
        The default behaviour is to call SetConnected() on all other
        connections in the call.
       */
-    virtual BOOL OnConnected(
+    virtual PBoolean OnConnected(
       OpalConnection & connection   ///<  Connection that indicates it is alerting
     );
 
     /**A call back function whenever a connection is "established".
-       This indicates that a connection to an endpoint was connected and that
-       media streams are opened. 
+       This indicates that a connection to an endpoint was established. This
+       usually occurs after OnConnected() and indicates that the connection
+       is both connected and has media flowing.
 
        In the context of H.323 this means that the CONNECT pdu has been
        received and either fast start was in operation or the subsequent Open
@@ -281,7 +223,7 @@ class OpalCall : public PSafeObject
        established and if so, marks the call as established and calls
        OnEstablishedCall().
       */
-    virtual BOOL OnEstablished(
+    virtual PBoolean OnEstablished(
       OpalConnection & connection   ///<  Connection that indicates it is alerting
     );
 
@@ -294,6 +236,18 @@ class OpalCall : public PSafeObject
       OpalConnection & connection   ///<  Connection that was established
     );
 
+    /**A call back function whenever a connection is "held" or "retrieved".
+       This indicates that a connection of a call was held, or
+       retrieved, either locally or by the remote endpoint.
+
+       The default behaviour does nothing.
+      */
+    virtual void OnHold(
+      OpalConnection & connection,   ///<  Connection that was held/retrieved
+      bool fromRemote,               ///<  Indicates remote has held local connection
+      bool onHold                    ///<  Indicates have just been held/retrieved.
+    );
+
     /**Get the other party's connection object.
        This will return the other party in the call. It will return NULL if
        there is no other party yet, or there are more than two parties in the
@@ -309,6 +263,52 @@ class OpalCall : public PSafeObject
       PINDEX idx,
       PSafetyMode mode = PSafeReference
     ) { return connectionsActive.GetAt(idx, mode); }
+
+    /**Find a connection of the specified class.
+       This searches the call for the Nth connection of the specified class.
+      */
+    template <class ConnClass>
+    PSafePtr<ConnClass> GetConnectionAs(
+      PINDEX count = 0,
+      PSafetyMode mode = PSafeReadWrite
+    )
+    {
+      PSafePtr<ConnClass> connection;
+      for (PSafePtr<OpalConnection> iterConn(connectionsActive, PSafeReference); iterConn != NULL; ++iterConn) {
+        if ((connection = PSafePtrCast<OpalConnection, ConnClass>(iterConn)) != NULL && count-- == 0) {
+          if (!connection.SetSafetyMode(mode))
+            connection.SetNULL();
+          break;
+        }
+      }
+      return connection;
+    }
+
+    /**Put call on hold.
+      */
+    bool Hold();
+
+    /**Retrieve call from hold.
+      */
+    bool Retrieve();
+
+    /**Indicate if call is in hold.
+      */
+    bool IsOnHold() const;
+
+    /**Transfer connection.
+       If the newAddress indicates a protocol type and it is the same as
+       connection then this simply calls TransferConnection on the
+       connection variable.
+
+       If there is a protocol change, e.g. "pc" to "t38", then the connection
+       is completely closed, all links to the second conection in the call (if
+       any) severed and a new connection establshed and new streams started.
+      */
+    bool Transfer(
+      OpalConnection & connection, ///< Connection to transfer
+      const PString & newAddress   ///< New address to transfer to
+    );
   //@}
 
   /**@name Media management */
@@ -321,43 +321,58 @@ class OpalCall : public PSafeObject
        This will also add to the list all media formats for which there are
        transcoders registered.
       */
-    OpalMediaFormatList GetMediaFormats(
+    virtual OpalMediaFormatList GetMediaFormats(
       const OpalConnection & connection,  ///<  Connection requesting formats
-      BOOL includeSpecifiedConnection     ///<  Include parameters media
+      PBoolean includeSpecifiedConnection     ///<  Include parameters media
     );
 
-    /**Open transmitter media streams for each connection.
-       All connections in the call except the one specified are requested to
-       open a source media stream. If successful, then PatchMediaStreams() is
-       called which in turns starts the sink media stream on the connection.
+    /**Open source media streams for the specified connection.
+       A source media stream is opened for the connection, if successful then
+       sink media streams are created for every other connection in the call.
+       If at least one sink is created then an OpalMediaPatch is created to
+       transfer data from the source to the sinks.
+
+       If session ID is zero a new session is created. If session ID is non
+       zero then that existing session is replaced.
       */
-    virtual BOOL OpenSourceMediaStreams(
-      const OpalConnection & connection,        ///<  Connection requesting open
-      const OpalMediaFormatList & mediaFormats, ///<  Optional media format to open
-      unsigned sessionID                        ///<  Session to start streams on
+    virtual bool OpenSourceMediaStreams(
+      OpalConnection & connection,              ///<  Connection requesting open
+      const OpalMediaType & mediaType,          ///<  Media type of channel to open
+      unsigned sessionID = 0,                   ///<  Session to start streams on
+      const OpalMediaFormat & preselectedFormat = OpalMediaFormat()  ///< Format for source stream to choose from
     );
 
-    /**Connect up the media streams on the connections.
-       This creates a media patch and a sink media stream on the specified
-       connection for the specified source media stream. The created media
-       patch is a thread, but it is not started immediately.
-     */
-    virtual BOOL PatchMediaStreams(
-      const OpalConnection & connection, ///<  Source connection
-      OpalMediaStream & source           ///<  Source media stream to patch
+    /**Select media format pair from the source/destination list.
+
+       Default behavour calls OpalTranscoder::SelectFormats().
+      */
+    virtual bool SelectMediaFormats(
+      const OpalMediaFormatList & srcFormats, ///<  Names of possible source formats
+      const OpalMediaFormatList & dstFormats, ///<  Names of possible destination formats
+      const OpalMediaFormatList & allFormats, ///<  Master list of formats for merging options
+      OpalMediaFormat & srcFormat,            ///<  Selected source format to be used
+      OpalMediaFormat & dstFormat             ///<  Selected destination format to be used
+    ) const;
+
+    /**Callback from the RTP session for statistics monitoring.
+       This is called every so many packets on the transmitter and receiver
+       threads of the RTP session indicating that the statistics have been
+       updated.
+
+       The default behaviour does nothing.
+      */
+    virtual void OnRTPStatistics(
+      const OpalConnection & connection,  ///<  Connection for the channel
+      const RTP_Session & session         ///<  Session with statistics
     );
-    
+
     /**Close the media streams on the connections.
      */
     virtual void CloseMediaStreams();
     
-    /**Remove the media streams on the connections.
-     */
-    virtual void RemoveMediaStreams();
-
     /**See if the media can bypass the local host.
      */
-    virtual BOOL IsMediaBypassPossible(
+    virtual PBoolean IsMediaBypassPossible(
       const OpalConnection & connection,  ///<  Source connection
       unsigned sessionID                  ///<  Session ID for media channel
     ) const;
@@ -404,14 +419,14 @@ class OpalCall : public PSafeObject
        Note this will be available even after the A party connection has been
        released from the call.
      */
-    const PString & GetPartyA() const { return partyA; }
+    const PString & GetPartyA() const { return m_partyA; }
 
     /**Get the B party for the call.
        Note this will be available even after the B party connection has been
        released from the call. Also this will only be the first B party if the
        object represents a conference call with more that 2 parties.
      */
-    const PString & GetPartyB() const { return partyB; }
+    const PString & GetPartyB() const { return m_partyB; }
 
     /**Set the B party for a call.
        This is used when we wish to make two outgoing calls and bridge them.
@@ -420,23 +435,42 @@ class OpalCall : public PSafeObject
       */
     void SetPartyB(
       const PString & b
-    ) { partyB = b; }
+    ) { m_partyB = b; }
+
+    /**Get indication that A-Party is the network.
+       This will indicate if the call is "incoming" or "outgoing" by looking at
+       the type of the A-party connection.
+      */
+    bool IsNetworkOriginated() const;
 
     /**Get the time the call started.
      */
     const PTime & GetStartTime() const { return startTime; }
   //@}
 
+    virtual PBoolean StartRecording(const PFilePath & fn);
+    virtual bool IsRecording() const;
+    virtual void StopRecording();
+    void OnStopRecordAudio(const PString & callToken);
 
   protected:
+    void SetPartyNames();
+
+    bool EnumerateConnections(
+      PSafePtr<OpalConnection> & connection,
+      PSafetyMode mode,
+      const OpalConnection * skipConnection = NULL
+    ) const;
+
     OpalManager & manager;
 
     PString myToken;
 
-    PString partyA;
-    PString partyB;
+    PString m_partyA;
+    PString m_partyB;
     PTime   startTime;
-    BOOL    isEstablished;
+    bool    isEstablished;
+    bool    isClearing;
 
     OpalConnection::CallEndReason callEndReason;
 
@@ -444,7 +478,11 @@ class OpalCall : public PSafeObject
 
     PSyncPoint * endCallSyncPoint;
 
-  friend OpalConnection::OpalConnection(OpalCall & call, OpalEndPoint & ep, const PString & token);
+    
+  //use to add the connection to the call's connection list
+  friend OpalConnection::OpalConnection(OpalCall &, OpalEndPoint &, const PString &, unsigned int, OpalConnection::StringOptions *);
+  //use to remove the connection from the call's connection list
+  friend OpalConnection::~OpalConnection();
 };
 
 
