@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20745 $
+ * $Revision: 21004 $
  * $Author: rjongbloed $
- * $Date: 2008-08-28 02:20:52 +0200 (jeu 28 aoû 2008) $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifndef __OPAL_MEDIASTRM_H
@@ -36,9 +36,10 @@
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
+
 #include <ptclib/delaychan.h>
 
-#include <opal/buildopts.h>
 #include <opal/mediafmt.h>
 #include <opal/mediacmd.h>
 #include <ptlib/safecoll.h>
@@ -112,8 +113,9 @@ class OpalMediaStream : public PSafeObject
        The default behaviour updates the mediaFormat member variable and
        pases the value on to the OpalMediaPatch.
       */
-    virtual PBoolean UpdateMediaFormat(
-      const OpalMediaFormat & mediaFormat  ///<  New media format
+    virtual bool UpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat,  ///<  New media format
+      bool fromPatch = false                ///<  Is being called from OpalMediaPatch
     );
 
     /**Execute the command specified to the transcoder. The commands are
@@ -343,7 +345,7 @@ class OpalMediaStream : public PSafeObject
       */
     bool RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage);
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
   //@}
@@ -525,7 +527,7 @@ class OpalRTPMediaStream : public OpalMediaStream
     virtual RTP_Session & GetRtpSession() const
     { return rtpSession; }
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
   //@}
@@ -669,7 +671,7 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
 };
 
 
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
 
 /**This class describes a media stream that transfers data to/from a audio
    PSoundChannel.
@@ -727,7 +729,7 @@ class OpalAudioMediaStream : public OpalRawMediaStream
     PINDEX soundChannelBuffers;
 };
 
-#endif // P_AUDIO
+#endif // OPAL_PTLIB_AUDIO
 
 #if OPAL_VIDEO
 
