@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20745 $
+ * $Revision: 21004 $
  * $Author: rjongbloed $
- * $Date: 2008-08-28 02:20:52 +0200 (jeu 28 aoû 2008) $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifndef __OPAL_TRANSCODERS_H
@@ -322,7 +322,7 @@ class OpalTranscoder : public OpalMediaFormatPair
     virtual bool AcceptEmptyPayload() const  { return acceptEmptyPayload; }
     virtual bool AcceptOtherPayloads() const { return acceptOtherPayloads; }
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
   //@}
