@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20752 $
+ * $Revision: 21062 $
  * $Author: rjongbloed $
- * $Date: 2008-08-28 08:59:51 +0200 (jeu 28 aoû 2008) $
+ * $Date: 2008-09-18 11:41:40 +0000 (Thu, 18 Sep 2008) $
  */
 
 #ifndef __OPAL_SIPEP_H
@@ -40,6 +40,8 @@
 #include <ptlib.h>
 #endif
 
+#include <opal/buildopts.h>
+
 #include <ptclib/sockagg.h>
 #include <opal/rtpep.h>
 #include <sip/sipcon.h>
@@ -283,7 +285,14 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     /**Handle an incoming MESSAGE PDU.
       */
-    virtual void OnReceivedMESSAGE(
+    virtual bool OnReceivedMESSAGE(
+      OpalTransport & transport,
+      SIP_PDU & response
+    );
+
+    /**Handle an incoming OPTIONS PDU.
+      */
+    virtual bool OnReceivedOPTIONS(
       OpalTransport & transport,
       SIP_PDU & response
     );
@@ -364,7 +373,57 @@ class SIPEndPoint : public OpalRTPEndPoint
     /**Unregister all current registrations.
       */
     bool UnregisterAll();
-    
+
+    /**Returns PTrue if the given URL has been registered 
+     * (e.g.: 6001@seconix.com).
+     */
+    PBoolean IsRegistered(const PString & aor);
+
+    /** Returns the number of registered accounts.
+     */
+    unsigned GetRegistrationsCount() const { return activeSIPHandlers.GetCount(SIP_PDU::Method_REGISTER); }
+
+    /** Information provided on the registration status. */
+    struct RegistrationStatus {
+      PString              m_addressofRecord;   ///< Address of record for registration
+      bool                 m_wasRegistering;    ///< Was registering or unregistering
+      bool                 m_reRegistering;     ///< Was a registration refresh
+      SIP_PDU::StatusCodes m_reason;            ///< Reason for status change
+      OpalProductInfo      m_productInfo;       ///< Server product info from registrar if available.
+    };
+
+    /**Callback called when a registration to a SIP registrar status.
+     */
+    virtual void OnRegistrationStatus(
+      const RegistrationStatus & status   ///< Status of registration request
+    );
+
+    // For backward compatibility
+    virtual void OnRegistrationStatus(
+      const PString & aor,
+      PBoolean wasRegistering,
+      PBoolean reRegistering,
+      SIP_PDU::StatusCodes reason
+    );
+
+    /**Callback called when a registration to a SIP registrars fails.
+       Deprecated, maintained for backward compatibility, use OnRegistrationStatus().
+     */
+    virtual void OnRegistrationFailed(
+      const PString & aor,
+      SIP_PDU::StatusCodes reason,
+      PBoolean wasRegistering
+    );
+
+    /**Callback called when a registration or an unregistration is successful.
+       Deprecated, maintained for backward compatibility, use OnRegistrationStatus().
+     */
+    virtual void OnRegistered(
+      const PString & aor,
+      PBoolean wasRegistering
+    );
+
+
     /**Subscribe to a notifier. This function is asynchronous to permit
      * several subscriptions to occur at the same time.
      */
@@ -387,6 +446,37 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PString & to
     );
 
+    /**Unsubscribe all current subscriptions.
+      */
+    bool UnsubcribeAll(
+      SIPSubscribe::PredefinedPackages eventPackage
+    );
+    bool UnsubcribeAll(
+      const PString & eventPackage
+    );
+
+    /**Returns PTrue if the endpoint is subscribed to some
+     * event for the given to address.
+     */
+    PBoolean IsSubscribed(
+      const PString & eventPackage,
+      const PString & to
+    );
+
+    /** Returns the number of registered accounts.
+     */
+    unsigned GetSubscriptionCount(const PString & eventPackage) { return activeSIPHandlers.GetCount(SIP_PDU::Method_SUBSCRIBE, eventPackage); }
+
+    /**Callback called when a subscription to a SIP UA status changes.
+     */
+    virtual void OnSubscriptionStatus(
+      const PString & eventPackage, ///< Event package subscribed to
+      const SIPURL & uri,           ///< Target URI for the subscription.
+      bool wasSubscribing,          ///< Indication the subscribing or unsubscribing
+      bool reSubscribing,           ///< If subscribing then indication was refeshing subscription
+      SIP_PDU::StatusCodes reason   ///< Status of subscription
+    );
+
 
     /**Send a message to the given URL.
      */
@@ -420,56 +510,6 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PString & note
     );
 
-    /**Callback called when a registration to a SIP registrar status.
-     * The PBoolean indicates if the operation that failed was a REGISTER or 
-     * an (UN)REGISTER.
-     */
-    virtual void OnRegistrationStatus(
-      const PString & aor,
-      PBoolean wasRegistering,
-      PBoolean reRegistering,
-      SIP_PDU::StatusCodes reason
-    );
-
-    /**Callback called when a registration to a SIP registrars fails.
-     * The PBoolean indicates if the operation that failed was a REGISTER or 
-     * an (UN)REGISTER.
-     */
-    virtual void OnRegistrationFailed(
-      const PString & aor,
-      SIP_PDU::StatusCodes reason,
-      PBoolean wasRegistering
-    );
-
-    /**Callback called when a registration or an unregistration is successful.
-     * The PBoolean indicates if the operation that failed was a registration or
-     * not.
-     */
-    virtual void OnRegistered(
-      const PString & aor,
-      PBoolean wasRegistering
-    );
-
-    
-    /**Returns PTrue if the given URL has been registered 
-     * (e.g.: 6001@seconix.com).
-     */
-    PBoolean IsRegistered(const PString & aor);
-
-    /**Returns PTrue if the endpoint is subscribed to some
-     * event for the given to address.
-     */
-    PBoolean IsSubscribed(
-      const PString & eventPackage,
-      const PString & to
-    );
-
-
-    /** Returns the number of registered accounts.
-     */
-    unsigned GetRegistrationsCount () { return activeSIPHandlers.GetRegistrationsCount (); }
-    
-
     /**Callback called when a message sent by the endpoint didn't reach
      * its destination or when the proxy or remote endpoint returns
      * an error code.
@@ -617,6 +657,10 @@ class SIPEndPoint : public OpalRTPEndPoint
     void SetUserAgent(const PString & str) { userAgentString = str; }
 
 
+    /** Return a bit mask of the allowed SIP methods.
+      */
+    virtual unsigned GetAllowedMethods() const;
+
     PBoolean SendResponse(
       SIP_PDU::StatusCodes code, 
       OpalTransport & transport, 
@@ -697,6 +741,22 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     typedef PThreadPool<SIP_PDU_Work, SIP_PDU_Thread> SIPMainThreadPool;
     SIPMainThreadPool threadPool;
+
+    class InterfaceMonitor : public PInterfaceMonitorClient
+    {
+        PCLASSINFO(InterfaceMonitor, PInterfaceMonitorClient);
+      public:
+        InterfaceMonitor(SIPEndPoint & manager);
+        
+        virtual void OnAddInterface(const PIPSocket::InterfaceEntry & entry);
+        virtual void OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
+
+    protected:
+        SIPEndPoint & m_endpoint;
+    } m_interfaceMonitor;
+
+    friend void InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry & entry);
+    friend void InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
 };
 
 #endif // __OPAL_SIPEP_H
