@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20765 $
+ * $Revision: 21004 $
  * $Author: rjongbloed $
- * $Date: 2008-08-29 06:02:51 +0200 (ven 29 aoû 2008) $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifndef __OPAL_SIPPDU_H
@@ -36,6 +36,7 @@
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
 
 #include <ptclib/mime.h>
 #include <ptclib/url.h>
@@ -654,7 +655,7 @@ class SIP_PDU : public PSafeObject
 
     /**Set mime allow field to all supported methods.
       */
-    void SetAllow(void);
+    void SetAllow(unsigned bitmask);
 
     /**Update the VIA field following RFC3261, 18.2.1 and RFC3581.
       */
@@ -837,14 +838,10 @@ class SIPInvite : public SIPTransaction
 {
     PCLASSINFO(SIPInvite, SIPTransaction);
   public:
-    SIPInvite(
-      SIPConnection & connection,
-      OpalTransport & transport
-    );
     SIPInvite(
       SIPConnection & connection,
       OpalTransport & transport,
-      OpalRTPSessionManager & sm
+      OpalRTPSessionManager * sm
     );
 
     virtual PBoolean OnReceivedResponse(SIP_PDU & response);
@@ -900,6 +897,7 @@ class SIPSubscribe : public SIPTransaction
       Presence,
       NumPredefinedPackages
     };
+    static PredefinedPackages GetEventPackage(const PString & name);
     static PString GetEventPackageName(PredefinedPackages pkg);
 
     struct Params {
