@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 19710 $
+ * $Revision: 21111 $
  * $Author: rjongbloed $
- * $Date: 2008-03-10 08:57:29 +0100 (lun 10 mar 2008) $
+ * $Date: 2008-09-22 01:32:48 +0000 (Mon, 22 Sep 2008) $
  */
 
 #include "precompile.h"
@@ -36,11 +36,11 @@
 PCREATE_PROCESS(OpalGw);
 
 
-const WORD DefaultHTTPPort = 6725;
+const WORD DefaultHTTPPort = 1719;
 static const char UsernameKey[] = "Username";
 static const char PasswordKey[] = "Password";
 static const char LogLevelKey[] = "Log Level";
-#if P_SSL
+#if OPAL_PTLIB_SSL
 static const char HTTPCertificateFileKey[]  = "HTTP Certificate";
 #endif
 static const char HttpPortKey[] = "HTTP Port";
@@ -61,17 +61,18 @@ static const char DisableFastStartKey[] = "Disable Fast Start";
 static const char DisableH245TunnelingKey[] = "Disable H.245 Tunneling";
 static const char DisableH245inSetupKey[] = "Disable H.245 in Setup";
 static const char H323BandwidthKey[] = "H.323 Bandwidth";
-static const char H323ListenersKey[] = "H.323 Listener Interfaces";
-static const char GatekeeperAddressKey[] = "Gatekeeper Address";
-static const char GatekeeperIdentifierKey[] = "Gatekeeper Identifier";
-static const char GatekeeperInterfaceKey[] = "Gatekeeper Interface";
-static const char GatekeeperPasswordKey[] = "Gatekeeper Password";
-static const char GatekeeperTokenOIDKey[] = "Gatekeeper Token OID";
+static const char H323ListenersKey[] = "H.323 Interfaces";
+static const char GatekeeperEnableKey[] = "Remote Gatekeeper Enable";
+static const char GatekeeperAddressKey[] = "Remote Gatekeeper Address";
+static const char GatekeeperIdentifierKey[] = "Remote Gatekeeper Identifier";
+static const char GatekeeperInterfaceKey[] = "Remote Gatekeeper Interface";
+static const char GatekeeperPasswordKey[] = "Remote Gatekeeper Password";
+static const char GatekeeperTokenOIDKey[] = "Remote Gatekeeper Token OID";
 
 static const char SIPUsernameKey[] = "SIP User Name";
 static const char SIPProxyKey[] = "SIP Proxy URL";
 static const char SIPRegistrarKey[] = "SIP Registrar";
-static const char SIPListenersKey[] = "SIP Listener Interfaces";
+static const char SIPListenersKey[] = "SIP Interfaces";
 
 static const char LIDKey[] = "Line Interface Devices";
 
@@ -181,12 +182,14 @@ PBoolean OpalGw::Initialise(const char * initMsg)
                                   GetLogLevel(),
                                   "1=Fatal only, 2=Errors, 3=Warnings, 4=Info, 5=Debug"));
 
-#if P_SSL
+#if OPAL_PTLIB_SSL
   // SSL certificate file.
   PString certificateFile = cfg.GetString(HTTPCertificateFileKey, "server.pem");
   rsrc->Add(new PHTTPStringField(HTTPCertificateFileKey, 25, certificateFile));
-  if (!SetServerCertificate(certificateFile, PTrue)) {
-    PSYSTEMLOG(Fatal, "BMAC\tCould not load certificate \"" << certificateFile << '"');
+  if (certificateFile.IsEmpty())
+    disableSSL = true;
+  else if (!SetServerCertificate(certificateFile, PTrue)) {
+    PSYSTEMLOG(Fatal, "OpalGw\tCould not load certificate \"" << certificateFile << '"');
     return PFalse;
   }
 #endif
@@ -266,7 +269,7 @@ MyManager::MyManager()
   sipEP = NULL;
 #endif
   potsEP = NULL;
-#if P_EXPAT
+#if OPAL_PTLIB_EXPAT
   ivrEP = NULL;
 #endif
 
@@ -306,12 +309,12 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
     sipEP = new SIPEndPoint(*this);
 #endif
 
-#ifdef OPAL_LID
+#if OPAL_LID
   if (potsEP == NULL)
     potsEP = new OpalLineEndPoint(*this);
 #endif
 
-#if P_EXPAT
+#if OPAL_PTLIB_EXPAT
   if (ivrEP == NULL)
     ivrEP = new OpalIVREndPoint(*this);
 #endif
@@ -321,9 +324,8 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   PStringArray formats = fieldArray->GetStrings(cfg);
   if (formats.GetSize() > 0)
     SetMediaFormatOrder(formats);
-  else {
+  else
     fieldArray->SetStrings(cfg, GetMediaFormatOrder());
-  }
   rsrc->Add(fieldArray);
 
   fieldArray = new PHTTPFieldArray(new PHTTPStringField(RemovedMediaKey, 25), PTrue);
@@ -364,6 +366,7 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
     for (PINDEX i = 1; i < aliases.GetSize(); i++)
       h323EP->AddAliasName(aliases[i]);
   }
+  rsrc->Add(fieldArray);
 
   h323EP->DisableFastStart(cfg.GetBoolean(DisableFastStartKey, h323EP->IsFastStartDisabled()));
   rsrc->Add(new PHTTPBooleanField(DisableFastStartKey,  h323EP->IsFastStartDisabled()));
@@ -381,6 +384,10 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   if (!h323EP->StartListeners(fieldArray->GetStrings(cfg))) {
     PSYSTEMLOG(Error, "Could not open any H.323 listeners!");
   }
+  rsrc->Add(fieldArray);
+
+  bool gkEnable = cfg.GetBoolean(GatekeeperEnableKey, true);
+  rsrc->Add(new PHTTPBooleanField(GatekeeperEnableKey, gkEnable));
 
   PString gkAddress = cfg.GetString(GatekeeperAddressKey);
   rsrc->Add(new PHTTPStringField(GatekeeperAddressKey, 25, gkAddress));
@@ -399,11 +406,17 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   h323EP->SetGkAccessTokenOID(cfg.GetString(GatekeeperTokenOIDKey));
   rsrc->Add(new PHTTPStringField(GatekeeperTokenOIDKey, 25, h323EP->GetGkAccessTokenOID()));
 
-  if (h323EP->UseGatekeeper(gkAddress, gkIdentifier, gkInterface)) {
-    PSYSTEMLOG(Info, "Register with gatekeeper " << *h323EP->GetGatekeeper());
+  if (gkEnable) {
+    if (h323EP->UseGatekeeper(gkAddress, gkIdentifier, gkInterface)) {
+      PSYSTEMLOG(Info, "Register with gatekeeper " << *h323EP->GetGatekeeper());
+    }
+    else {
+      PSYSTEMLOG(Error, "Could not register with gatekeeper!");
+    }
   }
   else {
-    PSYSTEMLOG(Error, "Could not register with gatekeeper!");
+    PSYSTEMLOG(Info, "Not using gatekeeper.");
+    h323EP->RemoveGatekeeper();
   }
 
   if (!gkServer->Initialise(cfg, rsrc))
@@ -426,6 +439,7 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   if (!sipEP->StartListeners(fieldArray->GetStrings(cfg))) {
     PSYSTEMLOG(Error, "Could not open any SIP listeners!");
   }
+  rsrc->Add(fieldArray);
 
   if (!registrar) {
     if (sipEP->Register(registrar)) {
@@ -440,14 +454,14 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
 
   // Add POTS and PSTN endpoints
   fieldArray = new PHTTPFieldArray(new PHTTPSelectField(LIDKey, OpalLineInterfaceDevice::GetAllDevices()), PFalse);
-  rsrc->Add(fieldArray);
   PStringArray devices = fieldArray->GetStrings(cfg);
   if (!potsEP->AddDeviceNames(devices)) {
     PSYSTEMLOG(Error, "No LID devices!");
   }
+  rsrc->Add(fieldArray);
 
 
-#if P_EXPAT
+#if OPAL_PTLIB_EXPAT
   // Create IVR protocol handler
   PString vxml = cfg.GetString(VXMLKey);
   rsrc->Add(new PHTTPStringField(VXMLKey, 25, vxml));
@@ -494,8 +508,8 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
 #endif
   
   fieldArray = new PHTTPFieldArray(new PHTTPStringField(DialPeerKey, 25), PTrue);
-  rsrc->Add(fieldArray);
   PStringArray routes = fieldArray->GetStrings(cfg);
+  rsrc->Add(fieldArray);
 
   if (!ivrAlias)
     routes += ".*:" + ivrAlias + "  = ivr:";
