@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20627 $
+ * $Revision: 21032 $
  * $Author: rjongbloed $
- * $Date: 2008-07-28 03:58:31 +0200 (lun 28 jui 2008) $
+ * $Date: 2008-09-17 06:23:59 +0000 (Wed, 17 Sep 2008) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -36,15 +36,16 @@
 #include <ptlib.h>
 #endif
 
-#ifndef P_AUDIO
-#error Cannot compile without PTLib sound channel support!
-#endif
-
 
 #include <wx/wx.h>
 #include <wx/dataobj.h>
 
 #include <opal/manager.h>
+
+#ifndef OPAL_PTLIB_AUDIO
+#error Cannot compile without PTLib sound channel support!
+#endif
+
 #include <opal/pcss.h>
 
 #ifndef OPAL_H323
@@ -84,23 +85,43 @@ class PwxString : public wxString
   public:
     PwxString() { }
     PwxString(const wxString & str) : wxString(str) { }
-    PwxString(const PString & str) : wxString((const char *)str) { }
-    PwxString(const char * str) : wxString(str) { }
+    PwxString(const PString & str) : wxString(str, wxConvUTF8 ) { }
+    PwxString(const PFilePath & fn) : wxString(fn, wxConvUTF8 ) { }
+    PwxString(const char * str) : wxString(str, wxConvUTF8) { }
+    PwxString(const OpalMediaFormat & fmt) : wxString(fmt, wxConvUTF8) { }
+#if wxUSE_UNICODE
+    PwxString(const wchar_t * wstr) : wxString(wstr) { }
+#endif
 
-    PwxString & operator=(const wxString & str) { wxString::operator=(str); return *this; }
-    PwxString & operator=(const PString & str) { wxString::operator=((const char *)str); return *this; }
-    PwxString & operator=(const char * str) { wxString::operator=(str); return *this; }
+    inline PwxString & operator=(const char * str)     { *this = wxString::wxString(str, wxConvUTF8); return *this; }
+#if wxUSE_UNICODE
+    inline PwxString & operator=(const wchar_t * wstr) { wxString::operator=(wstr); return *this; }
+#endif
+    inline PwxString & operator=(const wxString & str) { wxString::operator=(str); return *this; }
+    inline PwxString & operator=(const PString & str)  { *this = wxString::wxString(str, wxConvUTF8); return *this; }
 
-    bool operator==(const char * other)            const { return IsSameAs(other); }
-    bool operator==(const wxString & other)        const { return IsSameAs(other); }
-    bool operator==(const PString & other)         const { return IsSameAs((const char *)other); }
-    bool operator==(const PwxString & other)       const { return IsSameAs((const char *)other); }
-    bool operator==(const OpalMediaFormat & other) const { return IsSameAs((const char *)other); }
+    inline bool operator==(const char * other)            const { return IsSameAs(wxString(other, wxConvUTF8)); }
+#if wxUSE_UNICODE
+    inline bool operator==(const wchar_t * other)         const { return IsSameAs(other); }
+#endif
+    inline bool operator==(const wxString & other)        const { return IsSameAs(other); }
+    inline bool operator==(const PString & other)         const { return IsSameAs(wxString(other, wxConvUTF8)); }
+    inline bool operator==(const PwxString & other)       const { return IsSameAs(other); }
+    inline bool operator==(const OpalMediaFormat & other) const { return IsSameAs(wxString(other, wxConvUTF8)); }
 
-    operator PString() const { return c_str(); }
-    operator PIPSocket::Address() const { return PIPSocket::Address(PString(c_str())); }
-    friend ostream & operator<<(ostream & stream, const PwxString & string) { return stream << string.c_str(); }
+    inline bool operator!=(const char * other)            const { return !IsSameAs(wxString(other, wxConvUTF8)); }
+#if wxUSE_UNICODE
+    inline bool operator!=(const wchar_t * other)         const { return !IsSameAs(other); }
+#endif
+    inline bool operator!=(const wxString & other)        const { return !IsSameAs(other); }
+    inline bool operator!=(const PString & other)         const { return !IsSameAs(wxString(other, wxConvUTF8)); }
+    inline bool operator!=(const PwxString & other)       const { return !IsSameAs(other); }
+    inline bool operator!=(const OpalMediaFormat & other) const { return !IsSameAs(wxString(other, wxConvUTF8)); }
 
+    inline operator PString() const { return c_str(); }
+    inline operator PFilePath() const { return PFilePath(c_str()); }
+    inline operator PIPSocket::Address() const { return PIPSocket::Address(PString(c_str())); }
+    inline friend ostream & operator<<(ostream & stream, const PwxString & string) { return stream << string.c_str(); }
 };
 
 
@@ -118,7 +139,7 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
     MyManager & m_manager;
 };
 
-
+#if OPAL_H323
 class MyH323EndPoint : public H323EndPoint
 {
   public:
@@ -129,7 +150,7 @@ class MyH323EndPoint : public H323EndPoint
 
     MyManager & m_manager;
 };
-
+#endif
 
 class MySIPEndPoint : public SIPEndPoint
 {
@@ -143,6 +164,13 @@ class MySIPEndPoint : public SIPEndPoint
       PBoolean reRegistering,
       SIP_PDU::StatusCodes reason
     );
+    virtual void OnSubscriptionStatus(
+      const PString & eventPackage,
+      const SIPURL & uri,
+      bool wasSubscribing,
+      bool reSubscribing,
+      SIP_PDU::StatusCodes reason
+    );
 
     MyManager & m_manager;
 };
@@ -214,7 +242,7 @@ enum StatisticsPages {
 
 struct StatisticsField
 {
-  StatisticsField(const char * name, StatisticsPages page);
+  StatisticsField(const wxChar * name, StatisticsPages page);
   void Init(wxWindow * panel);
   void Clear();
   double CalculateBandwidth(DWORD bytes);
@@ -222,7 +250,7 @@ struct StatisticsField
   virtual void Update(const OpalConnection & connection, const OpalMediaStream & stream);
   virtual void GetValue(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics, wxString & value) = 0;
 
-  const char    * m_name;
+  const wxChar    * m_name;
   StatisticsPages m_page;
   wxStaticText  * m_staticText;
   wxString        m_printFormat;
@@ -347,25 +375,33 @@ class RegistrarInfo
     RegistrarInfo()
       : m_Active(false)
       , m_TimeToLive(300)
+      , m_MWI(true)
+      , m_Presence(true)
     {
     }
 
     bool operator==(const RegistrarInfo & other) const
     {
       return m_Active     == other.m_Active &&
-             m_Domain     == other.m_Domain &&
              m_User       == other.m_User &&
+             m_Domain     == other.m_Domain &&
+             m_AuthID     == other.m_AuthID &&
              m_Password   == other.m_Password &&
              m_TimeToLive == other.m_TimeToLive &&
-             m_Proxy      == other.m_Proxy;
+             m_Proxy      == other.m_Proxy &&
+             m_MWI        == other.m_MWI &&
+             m_Presence   == other.m_Presence;
     }
 
     bool      m_Active;
-    PwxString m_Domain;
     PwxString m_User;
+    PwxString m_Domain;
+    PwxString m_AuthID;
     PwxString m_Password;
     int       m_TimeToLive;
     PwxString m_Proxy;
+    bool      m_MWI;
+    bool      m_Presence;
 };
 
 typedef list<RegistrarInfo> RegistrarList;
@@ -534,11 +570,14 @@ class OptionsDialog : public wxDialog
     wxButton   * m_AddRegistrar;
     wxButton   * m_ChangeRegistrar;
     wxButton   * m_RemoveRegistrar;
-    wxTextCtrl * m_RegistrarDomain;
     wxTextCtrl * m_RegistrarUser;
+    wxTextCtrl * m_RegistrarDomain;
+    wxTextCtrl * m_RegistrarAuthID;
     wxTextCtrl * m_RegistrarPassword;
     wxSpinCtrl * m_RegistrarTimeToLive;
     wxCheckBox * m_RegistrarActive;
+    wxCheckBox * m_SubscribeMWI;
+    wxCheckBox * m_SubscribePresence;
 
     void FieldsToRegistrar(RegistrarInfo & info);
     void RegistrarToList(bool overwrite, RegistrarInfo * registrar, int position);
@@ -598,15 +637,15 @@ class MyMedia
 public:
   MyMedia();
   MyMedia(
-    const char * source,
+    const wxChar * source,
     const PString & format
   );
 
   bool operator<(const MyMedia & other) { return preferenceOrder < other.preferenceOrder; }
 
-  const char    * sourceProtocol;
+  const wxChar    * sourceProtocol;
   OpalMediaFormat mediaFormat;
-  const char    * validProtocols;
+  const wxChar    * validProtocols;
   int             preferenceOrder;
   bool            dirty;
 };
@@ -789,10 +828,12 @@ class MyManager : public wxFrame, public OpalManager
     PStringArray m_LocalInterfaces;
     void StartAllListeners();
 
+#if OPAL_H323
     MyH323EndPoint * h323EP;
     int              m_gatekeeperMode;
     PwxString        m_gatekeeperAddress;
     PwxString        m_gatekeeperIdentifier;
+#endif
     bool StartGatekeeper();
 
     MySIPEndPoint * sipEP;
@@ -824,7 +865,7 @@ class MyManager : public wxFrame, public OpalManager
     bool AdjustFrameSize();
 
     MyMediaList m_mediaInfo;
-    void InitMediaInfo(const char * source, const OpalMediaFormatList & formats);
+    void InitMediaInfo(const wxChar * source, const OpalMediaFormatList & formats);
     void ApplyMediaInfo();
 
 #if PTRACING
