@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 19279 $
+ * $Revision: 21117 $
  * $Author: rjongbloed $
- * $Date: 2008-01-17 05:08:34 +0100 (jeu 17 jan 2008) $
+ * $Date: 2008-09-22 03:13:07 +0000 (Mon, 22 Sep 2008) $
  */
 
 #ifndef _OpenPhone_VERSION_H
@@ -39,9 +39,9 @@
 
 
 #define MAJOR_VERSION 2
-#define MINOR_VERSION 1
-#define BUILD_TYPE    AlphaCode
-#define BUILD_NUMBER 0
+#define MINOR_VERSION 4
+#define BUILD_TYPE    ReleaseCode
+#define BUILD_NUMBER 1
 
 
 #endif  // _OpenPhone_VERSION_H
