@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20722 $
+ * $Revision: 20881 $
  * $Author: rjongbloed $
- * $Date: 2008-08-22 08:33:16 +0200 (ven 22 aoû 2008) $
+ * $Date: 2008-09-07 08:13:03 +0000 (Sun, 07 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -341,9 +341,6 @@ PBoolean H323UnidirectionalChannel::Open()
   if (PAssertNULL(mediaStream) == NULL)
     return false;
 
-  if (!H323Channel::Open())
-    return false;
-
   OpalCall & call = connection.GetCall();
 
   bool ok;
@@ -356,7 +353,7 @@ PBoolean H323UnidirectionalChannel::Open()
 
   if (ok) {
     capability->SetMediaFormatOptions(mediaStream->GetMediaFormat());
-    return true;
+    return H323Channel::Open();
   }
 
   PTRACE(1, "LogChan\t" << (GetDirection() == IsReceiver ? "Receive" : "Transmit")
