@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20759 $
- * $Author: hfriederich $
- * $Date: 2008-08-28 17:44:35 +0200 (jeu 28 aoû 2008) $
+ * $Revision: 21115 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-22 03:00:58 +0000 (Mon, 22 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -47,7 +47,7 @@
 #include <h323/h323pdu.h>
 #include <h323/h323rtp.h>
 
-#ifdef OPAL_H460
+#if OPAL_H460
 #include <h460/h4601.h>
 #endif
 
@@ -59,7 +59,7 @@ static class PAuthInitialiseInstantiateMe
     PAuthInitialiseInstantiateMe()
     {
       PWLibStupidLinkerHacks::h235AuthLoader = 1;
-#if P_SSL
+#if OPAL_PTLIB_SSL
       PWLibStupidLinkerHacks::h235AuthProcedure1Loader = 1;
 #endif
     }
@@ -90,7 +90,7 @@ H323Gatekeeper::H323Gatekeeper(H323EndPoint & ep, H323Transport * trans)
 #endif
   , requestMutex(1, 1)
   , authenticators(ep.CreateAuthenticators())
-#ifdef OPAL_H460
+#if OPAL_H460
   , features(ep.GetFeatureSet())
 #endif
 {
@@ -112,7 +112,7 @@ H323Gatekeeper::H323Gatekeeper(H323EndPoint & ep, H323Transport * trans)
 
   monitor = PThread::Create(PCREATE_NOTIFIER(MonitorMain), "GkMonitor");
   
-#ifdef OPAL_H460
+#if OPAL_H460
   features->AttachEndPoint(&ep);
   features->LoadFeatureSet(H460_Feature::FeatureRas);
 #endif
@@ -148,21 +148,21 @@ PString H323Gatekeeper::GetName() const
 PBoolean H323Gatekeeper::DiscoverAny()
 {
   gatekeeperIdentifier = PString();
-  return StartDiscovery(H323TransportAddress());
+  return StartGatekeeper(H323TransportAddress());
 }
 
 
 PBoolean H323Gatekeeper::DiscoverByName(const PString & identifier)
 {
   gatekeeperIdentifier = identifier;
-  return StartDiscovery(H323TransportAddress());
+  return StartGatekeeper(H323TransportAddress());
 }
 
 
 PBoolean H323Gatekeeper::DiscoverByAddress(const H323TransportAddress & address)
 {
   gatekeeperIdentifier = PString();
-  return StartDiscovery(address);
+  return StartGatekeeper(address);
 }
 
 
@@ -170,7 +170,7 @@ PBoolean H323Gatekeeper::DiscoverByNameAndAddress(const PString & identifier,
                                               const H323TransportAddress & address)
 {
   gatekeeperIdentifier = identifier;
-  return StartDiscovery(address);
+  return StartGatekeeper(address);
 }
 
 
@@ -196,93 +196,44 @@ static PBoolean WriteGRQ(H323Transport & transport, void * param)
 }
 
 
-PBoolean H323Gatekeeper::StartDiscovery(const H323TransportAddress & initialAddress)
+bool H323Gatekeeper::StartGatekeeper(const H323TransportAddress & initialAddress)
 {
   if (PAssertNULL(transport) == NULL)
-    return PFalse;
+    return false;
 
-  PAssert(!transport->IsRunning(), "Cannot do discovery on running RAS channel");
+  PAssert(!transport->IsRunning(), "Cannot do initial discovery on running RAS channel");
 
   H323TransportAddress address = initialAddress;
   if (address.IsEmpty())
     address = "udp$*:1719";
 
   if (!transport->ConnectTo(address))
-    return PFalse;
+    return false;
 
-  /// don't send GRQ if not requested
-  if (!endpoint.GetSendGRQ() && !initialAddress.IsEmpty()) {
-    StartChannel();
-    PTRACE(3, "RAS\tSkipping gatekeeper discovery for " << initialAddress);
-    return PTrue;
-  }
+  if (!StartChannel())
+    return false;
 
-  discoveryComplete = PFalse;
-
-  H323RasPDU pdu;
-  Request request(SetupGatekeeperRequest(pdu), pdu);
-
-  request.responseInfo = &address;
-
-  requestsMutex.Wait();
-  requests.SetAt(request.sequenceNumber, &request);
-  requestsMutex.Signal();
-
-  for (unsigned retry = 0; retry < endpoint.GetGatekeeperRequestRetries(); retry++) {
-    if (!transport->WriteConnect(WriteGRQ, &pdu)) {
-      PTRACE(1, "RAS\tError writing discovery PDU: " << transport->GetErrorText());
-      break;
-    }
-
-    H323RasPDU response;
-    transport->SetReadTimeout(endpoint.GetGatekeeperRequestTimeout ());
-    if (response.Read(*transport) && HandleTransaction(response) && discoveryComplete)
-      break;
-  }
-
-  transport->SetInterface(transport->GetLastReceivedInterface());
-
-  if (discoveryComplete) {
-    PTRACE(3, "RAS\tGatekeeper discovered at: "
-           << transport->GetRemoteAddress()
-           << " (if=" << transport->GetLocalAddress() << ')');
-    StartChannel();
-  }
-
-  requestsMutex.Wait();
-  requests.SetAt(request.sequenceNumber, NULL);
-  requestsMutex.Signal();
-
-  return discoveryComplete;
+  reregisterNow = true;
+  monitorTickle.Signal();
+  return true;
 }
 
 
-bool H323Gatekeeper::DiscoverGatekeeper(const H323TransportAddress & address)
+bool H323Gatekeeper::DiscoverGatekeeper()
 {
   discoveryComplete = false;
   
   H323RasPDU pdu;
   Request request(SetupGatekeeperRequest(pdu), pdu);
   
-  H323TransportAddress addr = address;
-  request.responseInfo = &addr;
+  H323TransportAddress address = transport->GetRemoteAddress();
+  request.responseInfo = &address;
   
   requestsMutex.Wait();
   requests.SetAt(request.sequenceNumber, &request);
   requestsMutex.Signal();
   
-  bool ok = false;
-  
-  for (unsigned retry = 0; retry < endpoint.GetGatekeeperRequestRetries(); retry++) {
-    if (!transport->WriteConnect(WriteGRQ, &pdu)) {
-      PTRACE(1, "RAS\tError writing discovery PDU: " << transport->GetErrorText());
-      break;
-    }
-    
-    ok = request.Poll(*this);
-    if (ok)
-      break;
-  }
+  request.Poll(*this, endpoint.GetGatekeeperRequestRetries(), endpoint.GetGatekeeperRequestTimeout());
   
   transport->SetInterface(transport->GetLastReceivedInterface());
   
@@ -300,6 +251,21 @@ bool H323Gatekeeper::DiscoverGatekeeper(const H323TransportAddress & address)
 }
 
 
+PBoolean H323Gatekeeper::WriteTo(H323TransactionPDU & pdu, 
+                                 const H323TransportAddressArray & addresses, 
+                                 PBoolean callback)
+{
+  if (!discoveryComplete && pdu.GetPDU().GetTag() == H225_RasMessage::e_gatekeeperRequest) {
+    if (!transport->WriteConnect(WriteGRQ, &pdu.GetPDU())) {
+      PTRACE(1, "RAS\tError writing discovery PDU: " << transport->GetErrorText());
+      return PFalse;
+    }
+    return PTrue;
+  }
+  return H323Transactor::WriteTo(pdu, addresses, callback);
+}
+
+
 unsigned H323Gatekeeper::SetupGatekeeperRequest(H323RasPDU & request)
 {
   if (PAssertNULL(transport) == NULL)
@@ -360,12 +326,15 @@ PBoolean H323Gatekeeper::OnReceiveGatekeeperConfirm(const H225_GatekeeperConfirm
   }
 
   H323TransportAddress locatedAddress(gcf.m_rasAddress, "udp");
-  PTRACE(3, "RAS\tGatekeeper discovery found " << locatedAddress);
+  PTRACE(3, "RAS\tGatekeeper discovered at: "
+         << transport->GetRemoteAddress()
+         << " (if=" << transport->GetLocalAddress() << ')');
 
   if (!transport->SetRemoteAddress(locatedAddress)) {
     PTRACE(2, "RAS\tInvalid gatekeeper discovery address: \"" << locatedAddress << '"');
     return PFalse;
   }
+  transport->SetInterface(transport->GetLastReceivedInterface());
 
   if (gcf.HasOptionalField(H225_GatekeeperConfirm::e_alternateGatekeeper))
     SetAlternates(gcf.m_alternateGatekeeper, PFalse);
@@ -398,7 +367,7 @@ PBoolean H323Gatekeeper::OnReceiveGatekeeperReject(const H225_GatekeeperReject &
 }
 
 
-PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg)
+PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg, PBoolean didGkDiscovery)
 {
   if (PAssertNULL(transport) == NULL)
     return PFalse;
@@ -408,8 +377,9 @@ PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg)
   H323RasPDU pdu;
   H225_RegistrationRequest & rrq = pdu.BuildRegistrationRequest(GetNextSequenceNumber());
 
-  // If discoveryComplete flag is PFalse then do lightweight reregister
-  rrq.m_discoveryComplete = discoveryComplete;
+  // the discoveryComplete flag of the RRQ is not to be confused with the
+  // discoveryComplete member variable of this instance...
+  rrq.m_discoveryComplete = didGkDiscovery;
 
   rrq.m_rasAddress.SetSize(1);
   H323TransportAddress rasAddress = transport->GetLocalAddress();
@@ -418,16 +388,10 @@ PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg)
   PIPSocket::Address localAddress, remoteAddress;
   WORD localPort;
   
-  if(rasAddress.GetIpAndPort(localAddress, localPort) &&
-     transport->GetRemoteAddress().GetIpAddress(remoteAddress))
-  {
-      OpalManager & manager = transport->GetEndPoint().GetManager();
-      
-      if(manager.TranslateIPAddress(localAddress, remoteAddress))
-      {
-          rasAddress = H323TransportAddress(localAddress, localPort);
-      }
-  }
+  if (rasAddress.GetIpAndPort(localAddress, localPort) &&
+      transport->GetRemoteAddress().GetIpAddress(remoteAddress) &&
+      transport->GetEndPoint().GetManager().TranslateIPAddress(localAddress, remoteAddress))
+    rasAddress = H323TransportAddress(localAddress, localPort);
   
   rasAddress.SetPDU(rrq.m_rasAddress[0]);
 
@@ -513,14 +477,11 @@ PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg)
     rrq.m_callCreditCapability.m_canEnforceDurationLimit = PTrue;
   }
 
-  if (IsRegistered()) {
+  if (IsRegistered()) { // send lightweight RRQ
     rrq.IncludeOptionalField(H225_RegistrationRequest::e_keepAlive);
     rrq.m_keepAlive = PTrue;
   }
 
-  // After doing full register, do lightweight reregisters from now on
-  discoveryComplete = PFalse;
-
   Request request(rrq.m_requestSeqNum, pdu);
   if (MakeRequest(request))
     return PTrue;
@@ -580,6 +541,7 @@ PBoolean H323Gatekeeper::OnReceiveRegistrationConfirm(const H225_RegistrationCon
     return PFalse;
 
   registrationFailReason = RegistrationSuccessful;
+  reregisterNow = false;
 
   endpointIdentifier = rcf.m_endpointIdentifier;
   PTRACE(3, "RAS\tRegistered " << endpointIdentifier << " with " << gatekeeperIdentifier);
@@ -689,6 +651,25 @@ PBoolean H323Gatekeeper::OnReceiveRegistrationReject(const H225_RegistrationReje
 void H323Gatekeeper::RegistrationTimeToLive()
 {
   PTRACE(3, "RAS\tTime To Live reregistration");
+  
+  PBoolean didGkDiscovery = false;
+
+  if (!discoveryComplete) {
+    if (endpoint.GetSendGRQ()) {
+      if (DiscoverGatekeeper()) {
+        requiresDiscovery = false;
+        didGkDiscovery = true;
+      } else {
+        PTRACE_IF(2, !reregisterNow, "RAS\tDiscovery failed, retrying in 1 minute");
+        timeToLive = PTimeInterval(0, 0, 1);
+        return;
+      }
+    }
+    else {
+      PTRACE_IF(3, !requiresDiscovery, "RAS\tSkipping gatekeeper discovery for " << transport->GetRemoteAddress());
+      discoveryComplete = true;
+    }
+  }
 
   if (requiresDiscovery) {
     PTRACE(3, "RAS\tRepeating discovery on gatekeepers request.");
@@ -702,9 +683,10 @@ void H323Gatekeeper::RegistrationTimeToLive()
     }
 
     requiresDiscovery = PFalse;
+    didGkDiscovery = true;
   }
 
-  if (!RegistrationRequest(autoReregister)) {
+  if (!RegistrationRequest(autoReregister, didGkDiscovery)) {
     PTRACE_IF(2, !reregisterNow, "RAS\tTime To Live reregistration failed, retrying in 1 minute");
     timeToLive = PTimeInterval(0, 0, 1);
   }
@@ -825,6 +807,7 @@ PBoolean H323Gatekeeper::OnReceiveUnregistrationRequest(const H225_Unregistratio
 
   if (autoReregister) {
     PTRACE(4, "RAS\tReregistering by setting timeToLive");
+    discoveryComplete = PFalse;
     reregisterNow = PTrue;
     monitorTickle.Signal();
   }
@@ -1720,8 +1703,7 @@ void H323Gatekeeper::MonitorMain(PThread &, INT)
     if (monitorStop)
       break;
 
-    if (reregisterNow || 
-                (!timeToLive.IsRunning() && timeToLive.GetResetTime() > 0)) {
+    if (reregisterNow || (!timeToLive.IsRunning() && timeToLive.GetResetTime() > 0)) {
       RegistrationTimeToLive();
       timeToLive.Reset();
     }
@@ -1810,7 +1792,7 @@ void H323Gatekeeper::Connect(const H323TransportAddress & address,
                              const PString & gkid)
 {
   if (transport == NULL)
-    transport = new H323TransportUDP(endpoint, PIPSocket::GetDefaultIpAny());
+    transport = CreateTransport(PIPSocket::GetDefaultIpAny());
 
   transport->SetRemoteAddress(address);
   transport->Connect();
@@ -1871,7 +1853,7 @@ PBoolean H323Gatekeeper::MakeRequest(Request & request)
       transport->CleanUpOnTermination();
       delete transport;
 
-      transport = new H323TransportUDP(endpoint,localAddress,localPort);
+      transport = CreateTransport(localAddress,localPort);
       transport->SetRemoteAddress (altInfo->rasAddress);
       transport->Connect();
       gatekeeperIdentifier = altInfo->gatekeeperIdentifier;
@@ -1903,12 +1885,40 @@ PBoolean H323Gatekeeper::MakeRequest(Request & request)
 }
 
 
+H323Transport * H323Gatekeeper::CreateTransport(PIPSocket::Address binding, WORD port, PBoolean reuseAddr)
+{
+  return new H323TransportUDP(endpoint, binding, port, reuseAddr);
+}
+
+
 void H323Gatekeeper::OnAddInterface(const PIPSocket::InterfaceEntry & /*entry*/, PINDEX priority)
 {
-  if (priority == HighPriority)
-    return;
-  
-  UpdateConnectionStatus();
+  if (priority != HighPriority)
+    UpdateConnectionStatus();
+  else {
+    // special case if interface filtering is used: A new interface may 'hide' the old interface.
+    // If this is the case, remove the transport interface and do a full rediscovery. 
+    //
+    // There is a race condition: If the transport interface binding is cleared AFTER
+    // PMonitoredSockets::ReadFromSocket() is interrupted and starts listening again,
+    // the transport will still listen on the old interface only. Therefore, clear the
+    // socket binding BEFORE the monitored sockets update their interfaces.
+    if (PInterfaceMonitor::GetInstance().HasInterfaceFilter()) {
+      PString iface = transport->GetInterface();
+      if (iface.IsEmpty()) // not connected.
+        return;
+      
+      PIPSocket::Address addr;
+      if (!transport->GetRemoteAddress().GetIpAddress(addr))
+        return;
+      
+      PStringArray ifaces = lowPriorityMonitor.GetInterfaces(PFalse, addr);
+      
+      if (ifaces.GetStringsIndex(iface) == P_MAX_INDEX) { // original interface no longer available
+        transport->SetInterface(PString::Empty());
+      }
+    }
+  }
 }
 
 
@@ -1951,19 +1961,18 @@ void H323Gatekeeper::UpdateConnectionStatus()
   if (!transport->GetRemoteAddress().GetIpAddress(addr))
     return;
   
-  if (lowPriorityMonitor.GetInterfaces(FALSE, addr).GetSize() > 0) {
+  if (lowPriorityMonitor.GetInterfaces(false, addr).GetSize() > 0) {
     // at least one interface available, locate gatekeper
-    if (DiscoverGatekeeper(transport->GetRemoteAddress())) {
-      reregisterNow = PTrue;
-      monitorTickle.Signal();
-    }
+    discoveryComplete = PFalse;
+    reregisterNow = PTrue;
+    monitorTickle.Signal();
   }
 }
 
 
 PBoolean H323Gatekeeper::OnSendFeatureSet(unsigned pduType, H225_FeatureSet & message) const
 {
-#ifdef OPAL_H460
+#if OPAL_H460
   return features->SendFeature(pduType, message);
 #else
   return endpoint.OnSendFeatureSet(pduType, message);
@@ -1973,7 +1982,7 @@ PBoolean H323Gatekeeper::OnSendFeatureSet(unsigned pduType, H225_FeatureSet & me
 
 void H323Gatekeeper::OnReceiveFeatureSet(unsigned pduType, const H225_FeatureSet & message) const
 {
-#ifdef OPAL_H460
+#if OPAL_H460
   features->ReceiveFeature(pduType, message);
 #else
   endpoint.OnReceiveFeatureSet(pduType, message);
