@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20401 $
- * $Author: csoutheren $
- * $Date: 2008-06-05 07:00:41 +0200 (jeu 05 jun 2008) $
+ * $Revision: 21115 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-22 03:00:58 +0000 (Mon, 22 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -240,7 +240,8 @@ H323TransportAddressArray H323Transactor::GetInterfaceAddresses(PBoolean exclude
   else
     return OpalGetInterfaceAddresses(transport->GetLocalAddress(),
                                      excludeLocalHost,
-                                     associatedTransport);
+                                     associatedTransport,
+                                     NULL);
 }
 
 PBoolean H323Transactor::StartChannel()
@@ -548,21 +549,27 @@ H323Transactor::Request::Request(unsigned seqNum,
 }
 
 
-PBoolean H323Transactor::Request::Poll(H323Transactor & rasChannel)
+PBoolean H323Transactor::Request::Poll(H323Transactor & rasChannel, unsigned numRetries, PTimeInterval timeout)
 {
   H323EndPoint & endpoint = rasChannel.GetEndPoint();
 
   responseResult = AwaitingResponse;
+  
+  if (numRetries == 0)
+    numRetries = endpoint.GetRasRequestRetries();
+  
+  if (timeout == 0)
+    timeout = endpoint.GetRasRequestTimeout();
 
-  for (unsigned retry = 1; retry <= endpoint.GetRasRequestRetries(); retry++) {
+  for (unsigned retry = 1; retry <= numRetries; retry++) {
     // To avoid race condition with RIP must set timeout before sending the packet
-    whenResponseExpected = PTimer::Tick() + endpoint.GetRasRequestTimeout();
+    whenResponseExpected = PTimer::Tick() + timeout;
 
     if (!rasChannel.WriteTo(requestPDU, requestAddresses, PFalse))
       break;
 
     PTRACE(3, "Trans\tWaiting on response to seqnum=" << requestPDU.GetSequenceNumber()
-           << " for " << setprecision(1) << endpoint.GetRasRequestTimeout() << " seconds");
+           << " for " << setprecision(1) << timeout << " seconds");
 
     do {
       // Wait for a response
@@ -597,7 +604,7 @@ PBoolean H323Transactor::Request::Poll(H323Transactor & rasChannel)
     } while (responseResult == AwaitingResponse);
 
     PTRACE(1, "Trans\tTimeout on request seqnum=" << requestPDU.GetSequenceNumber()
-           << ", try #" << retry << " of " << endpoint.GetRasRequestRetries());
+           << ", try #" << retry << " of " << numRetries);
   }
 
   return PFalse;
@@ -902,7 +909,7 @@ PBoolean H323TransactionServer::AddListener(const H323TransportAddress & interfa
   for (i = 0; i < interfaces.GetSize(); i++) {
     addr = interfaces[i].GetAddress();
     if (addr != 0) {
-      if (AddListener(new H323TransportUDP(ownerEndPoint, addr, port)))
+      if (AddListener(new H323TransportUDP(ownerEndPoint, addr, port, false, true)))
         atLeastOne = PTrue;
     }
   }
