@@ -27,12 +27,13 @@
  *
  *
  *
- * $Revision: 20711 $
- * $Author: dereksmithies $
- * $Date: 2008-08-21 05:18:38 +0200 (jeu 21 aoû 2008) $
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
+#include <opal/buildopts.h>
 #include <typeinfo>
 
 
