@@ -28,12 +28,13 @@
  * The processor parts were abstracted out by  by Stephen Cook (sitiveni@gmail.com)
  * and this work was sponsored by the Google summer of code 
  *
- * $Revision: 20711 $
- * $Author: dereksmithies $
- * $Date: 2008-08-21 05:18:38 +0200 (jeu 21 aoû 2008) $
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
+#include <opal/buildopts.h>
 #include <typeinfo>
 
 #ifdef P_USE_PRAGMA
