@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20730 $
- * $Author: dsandras $
- * $Date: 2008-08-23 14:01:28 +0200 (sam 23 aoû 2008) $
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -35,6 +35,8 @@
 #pragma implementation "call.h"
 #endif
 
+#include <opal/buildopts.h>
+
 #include <opal/call.h>
 
 #include <opal/manager.h>
@@ -158,6 +160,8 @@ PBoolean OpalCall::OnSetUp(OpalConnection & connection)
   if (isClearing)
     return false;
 
+  SetPartyNames();
+
   bool ok = false;
 
   PSafePtr<OpalConnection> otherConnection;
@@ -166,8 +170,6 @@ PBoolean OpalCall::OnSetUp(OpalConnection & connection)
       ok = true;
   }
 
-  SetPartyNames();
-
   return ok;
 }
 
@@ -463,7 +465,8 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     if (sinkStream != NULL) {
       startedOne = true;
       if (patch == NULL) {
-        patch = manager.CreateMediaPatch(*sourceStream, sourceStream->RequiresPatchThread(sinkStream));
+        patch = manager.CreateMediaPatch(*sourceStream, sinkStream->RequiresPatchThread(sourceStream) &&
+                                                        sourceStream->RequiresPatchThread(sinkStream));
         if (patch == NULL)
           return false;
       }
@@ -643,7 +646,11 @@ void OpalCall::SetPartyNames()
   if (connectionA == NULL)
     return;
 
-  m_partyA = connectionA->IsNetworkConnection() ? connectionA->GetRemotePartyURL() : connectionA->GetLocalPartyURL();
+  bool networkA = connectionA->IsNetworkConnection();
+  if (networkA)
+    m_partyA = connectionA->GetRemotePartyURL();
+  if (!networkA || m_partyA.IsEmpty())
+    m_partyA = connectionA->GetLocalPartyURL();
 
   PSafePtr<OpalConnection> connectionB = connectionsActive.GetAt(1, PSafeReadOnly);
   if (connectionB == NULL)
@@ -651,19 +658,13 @@ void OpalCall::SetPartyNames()
 
   if (connectionB->IsNetworkConnection()) {
     m_partyB = connectionB->GetRemotePartyURL();
-    if (!connectionA->IsNetworkConnection()) {
-      connectionA->SetRemotePartyName(connectionB->GetRemotePartyName());
-      connectionA->SetRemotePartyAddress(connectionB->GetRemotePartyURL());
-      connectionA->SetProductInfo(connectionB->GetRemoteProductInfo());
-    }
+    if (!connectionA->IsNetworkConnection())
+      connectionA->CopyPartyNames(*connectionB);
   }
   else {
     m_partyB = connectionB->GetLocalPartyURL();
-    if (connectionA->IsNetworkConnection()) {
-      connectionB->SetRemotePartyName(connectionA->GetRemotePartyName());
-      connectionB->SetRemotePartyAddress(connectionA->GetRemotePartyURL());
-      connectionB->SetProductInfo(connectionA->GetRemoteProductInfo());
-    }
+    if (connectionA->IsNetworkConnection())
+      connectionB->CopyPartyNames(*connectionA);
   }
 }
 
