@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 20783 $
- * $Author: csoutheren $
- * $Date: 2008-08-31 06:28:53 +0200 (dim 31 aoû 2008) $
+ * $Revision: 21014 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 12:36:29 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -75,6 +75,7 @@ OpalMediaStream::OpalMediaStream(OpalConnection & conn, const OpalMediaFormat &
   , mediaPatch(NULL)
 {
   connection.SafeReference();
+  PTRACE(5, "Media\tCreated " << (isSource ? "Source" : "Sink") << ' ' << this);
 }
 
 
@@ -82,6 +83,7 @@ OpalMediaStream::~OpalMediaStream()
 {
   Close();
   connection.SafeDereference();
+  PTRACE(5, "Media\tDestroyed " << (isSource ? "Source" : "Sink") << ' ' << this);
 }
 
 
@@ -102,14 +104,14 @@ OpalMediaFormat OpalMediaStream::GetMediaFormat() const
 }
 
 
-PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
+PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaFormat, bool fromPatch)
 {
   PSafeLockReadWrite safeLock(*this);
   if (!safeLock.IsLocked())
     return false;
 
   // If we are source, then update the sink side, and vice versa
-  if (mediaPatch != NULL) {
+  if (mediaPatch != NULL && !fromPatch) {
     if (!mediaPatch->UpdateMediaFormat(newMediaFormat, IsSink())) {
       PTRACE(2, "Media\tPatch did not allow media format update of " << *this);
       return false;
@@ -415,8 +417,8 @@ void OpalMediaStream::EnableJitterBuffer() const
 
 void OpalMediaStream::SetPaused(bool p)
 {
+  PTRACE_IF(3, paused != p, "Media\t" << (p ? "Paused" : "Resumed") << " stream " << *this);
   paused = p;
-  PTRACE(3, "Media\t" << (p ? "Paused" : "Resumed") << " stream " << *this);
 }
 
 
@@ -457,7 +459,7 @@ PBoolean OpalMediaStream::RemoveFilter(const PNotifier & Filter, const OpalMedia
 }
 
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
 void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics) const
 {
   PSafeLockReadOnly safeLock(*this);
@@ -666,9 +668,11 @@ PBoolean OpalRTPMediaStream::SetDataSize(PINDEX dataSize)
      practical UDP packet size. This means we have a buffer that can accept
      whatever the RTP sender throws at us. For sink, we just clamp it to the
      maximum size based on MTU (or other criteria). */
-  PINDEX maxSize = IsSource() ? 2048 : connection.GetMaxRtpPayloadSize();
-  if (defaultDataSize < maxSize)
-    defaultDataSize = maxSize;
+  PINDEX minSize = IsSource() ? 2048 : connection.GetMaxRtpPayloadSize();
+  if (defaultDataSize < minSize) {
+    PTRACE(1, "Media\tClamping RTP media stream data size from " << dataSize << " to minimum " << minSize);
+    defaultDataSize = minSize;
+  }
 
   return true;
 }
@@ -688,7 +692,7 @@ void OpalRTPMediaStream::EnableJitterBuffer() const
 }
 
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
 void OpalRTPMediaStream::GetStatistics(OpalMediaStatistics & statistics) const
 {
   rtpSession.GetStatistics(statistics, IsSource());
@@ -877,7 +881,7 @@ PBoolean OpalFileMediaStream::WriteData(const BYTE * data, PINDEX length, PINDEX
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
 
 OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
                                            const OpalMediaFormat & mediaFormat,
@@ -917,10 +921,12 @@ PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize)
      sending an even number of our multiplier. */
   const unsigned MinTime = 20;
   PINDEX minSize = mediaFormat.GetClockRate()/(1000/MinTime)*sizeof(short);
-  if (dataSize < minSize)
-    dataSize = (minSize/dataSize + 1)*minSize;
+  if (dataSize < minSize) {
+    PTRACE(1, "Media\tClamping audio stream data size from " << dataSize << " to minimum " << minSize);
+    dataSize = ((minSize+dataSize-1)/dataSize)*dataSize;
+  }
 
-  PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to  "
+  PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to "
          << dataSize << " bytes and " << soundChannelBuffers << " buffers.");
   return OpalMediaStream::SetDataSize(dataSize) &&
          ((PSoundChannel *)channel)->SetBuffers(dataSize, soundChannelBuffers);
@@ -932,7 +938,7 @@ PBoolean OpalAudioMediaStream::IsSynchronous() const
   return true;
 }
 
-#endif // P_AUDIO
+#endif // OPAL_PTLIB_AUDIO
 
 
 ///////////////////////////////////////////////////////////////////////////////
