@@ -26,12 +26,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20808 $
- * $Author: csoutheren $
- * $Date: 2008-09-01 10:20:19 +0200 (lun 01 sep 2008) $
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
+
+#include <opal/buildopts.h>
+
 #include <opal.h>
 #include <opal/manager.h>
 #include <opal/pcss.h>
@@ -85,7 +88,7 @@ class OpalMessageBuffer
 #define SET_MESSAGE_STRING(msg, member, str) (msg).SetString(&(msg)->member, str)
 
 
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
 
 class OpalPCSSEndPoint_C : public OpalPCSSEndPoint
 {
@@ -99,7 +102,7 @@ class OpalPCSSEndPoint_C : public OpalPCSSEndPoint
     OpalManager_C & m_manager;
 };
 
-#endif // P_AUDIO
+#endif // OPAL_PTLIB_AUDIO
 
 
 class OpalLocalEndPoint_C : public OpalLocalEndPoint
@@ -130,10 +133,14 @@ class SIPEndPoint_C : public SIPEndPoint
     SIPEndPoint_C(OpalManager_C & manager);
 
     virtual void OnRegistrationStatus(
-      const PString & aor,
-      PBoolean wasRegistering,
-      PBoolean reRegistering,
-      SIP_PDU::StatusCodes reason
+      const RegistrationStatus & status
+    );
+    virtual void OnSubscriptionStatus(
+      const PString & eventPackage, ///< Event package subscribed to
+      const SIPURL & uri,           ///< Target URI for the subscription.
+      bool wasSubscribing,          ///< Indication the subscribing or unsubscribing
+      bool reSubscribing,           ///< If subscribing then indication was refeshing subscription
+      SIP_PDU::StatusCodes reason   ///< Status of subscription
     );
 
   private:
@@ -147,7 +154,7 @@ class OpalManager_C : public OpalManager
   public:
     OpalManager_C(unsigned version)
       : localEP(NULL)
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
       , pcssEP(NULL)
 #endif
       , m_apiVersion(version)
@@ -191,7 +198,7 @@ class OpalManager_C : public OpalManager
     void OnIndMediaStream(const OpalMediaStream & stream, OpalMediaStates state);
 
     OpalLocalEndPoint_C * localEP;
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
     OpalPCSSEndPoint_C  * pcssEP;
 #endif
 
@@ -331,6 +338,16 @@ OpalMessage * OpalMessageBuffer::Detach()
 }
 
 
+PString BuildProductName(const OpalProductInfo & info)
+{
+  if (info.comments.IsEmpty())
+    return info.name;
+  if (info.comments[0] == '(')
+    return info.name + ' ' + info.comments;
+  return info.name + " (" + info.comments + ')';
+}
+
+
 ///////////////////////////////////////
 
 OpalLocalEndPoint_C::OpalLocalEndPoint_C(OpalManager_C & mgr)
@@ -373,6 +390,15 @@ static void SetIncomingCall(OpalMessageBuffer & message, const OpalConnection &
   SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_calledAddress, network->GetCalledPartyURL());
   SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_calledPartyNumber, network->GetCalledPartyNumber());
 
+  const OpalProductInfo & info = connection.GetProductInfo();
+  SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_product.m_vendor,  info.vendor);
+  SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_product.m_name,    BuildProductName(info));
+  SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_product.m_version, info.version);
+
+  message->m_param.m_incomingCall.m_product.m_t35CountryCode   = info.t35CountryCode;
+  message->m_param.m_incomingCall.m_product.m_t35Extension     = info.t35Extension;
+  message->m_param.m_incomingCall.m_product.m_manufacturerCode = info.manufacturerCode;
+
   PTRACE(4, "OpalC API\tOpalIndIncomingCall: token=\""  << message->m_param.m_incomingCall.m_callToken << "\"\n"
             "  Local  - URL=\"" << message->m_param.m_incomingCall.m_localAddress << "\"\n"
             "  Remote - URL=\"" << message->m_param.m_incomingCall.m_remoteAddress << "\""
@@ -491,7 +517,7 @@ bool OpalLocalEndPoint_C::OnWriteMediaData(const OpalLocalConnection & connectio
 
 ///////////////////////////////////////
 
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
 
 OpalPCSSEndPoint_C::OpalPCSSEndPoint_C(OpalManager_C & mgr)
   : OpalPCSSEndPoint(mgr)
@@ -526,7 +552,7 @@ PBoolean OpalPCSSEndPoint_C::OnShowOutgoing(const OpalPCSSConnection & connectio
   return true;
 }
 
-#endif // P_AUDIO
+#endif // OPAL_PTLIB_AUDIO
 
 
 ///////////////////////////////////////
@@ -540,35 +566,62 @@ SIPEndPoint_C::SIPEndPoint_C(OpalManager_C & mgr)
 }
 
 
-void SIPEndPoint_C::OnRegistrationStatus(const PString & aor,
-                                         PBoolean wasRegistering,
-                                         PBoolean reRegistering,
-                                         SIP_PDU::StatusCodes reason)
+void SIPEndPoint_C::OnRegistrationStatus(const RegistrationStatus & status)
 {
   OpalMessageBuffer message(OpalIndRegistration);
   SET_MESSAGE_STRING(message, m_param.m_registrationStatus.m_protocol, OPAL_PREFIX_SIP);
-  SET_MESSAGE_STRING(message, m_param.m_registrationStatus.m_serverName, aor);
-  if (reason == SIP_PDU::Information_Trying)
+  SET_MESSAGE_STRING(message, m_param.m_registrationStatus.m_serverName, status.m_addressofRecord);
+
+  SET_MESSAGE_STRING(message, m_param.m_registrationStatus.m_product.m_vendor,  status.m_productInfo.vendor);
+  SET_MESSAGE_STRING(message, m_param.m_registrationStatus.m_product.m_name,    BuildProductName(status.m_productInfo));
+  SET_MESSAGE_STRING(message, m_param.m_registrationStatus.m_product.m_version, status.m_productInfo.version);
+
+  message->m_param.m_registrationStatus.m_product.m_t35CountryCode   = status.m_productInfo.t35CountryCode;
+  message->m_param.m_registrationStatus.m_product.m_t35Extension     = status.m_productInfo.t35Extension;
+  message->m_param.m_registrationStatus.m_product.m_manufacturerCode = status.m_productInfo.manufacturerCode;
+
+  if (status.m_reason == SIP_PDU::Information_Trying)
     message->m_param.m_registrationStatus.m_status = OpalRegisterRetrying;
-  else if (reason/100 == 2) {
-    if (wasRegistering)
-      message->m_param.m_registrationStatus.m_status = reRegistering ? OpalRegisterRestored : OpalRegisterSuccessful;
+  else if (status.m_reason/100 == 2) {
+    if (status.m_wasRegistering)
+      message->m_param.m_registrationStatus.m_status = status.m_reRegistering ? OpalRegisterRestored : OpalRegisterSuccessful;
     else
       message->m_param.m_registrationStatus.m_status = OpalRegisterRemoved;
   }
   else {
     PStringStream strm;
-    strm << "Error " << reason << " in SIP ";
-    if (!wasRegistering)
+    strm << "Error " << status.m_reason << " in SIP ";
+    if (!status.m_wasRegistering)
       strm << "un";
     strm << "registration.";
     SET_MESSAGE_STRING(message, m_param.m_registrationStatus.m_error, strm);
-    message->m_param.m_registrationStatus.m_status = wasRegistering ? OpalRegisterFailed : OpalRegisterRemoved;
+    message->m_param.m_registrationStatus.m_status = status.m_wasRegistering ? OpalRegisterFailed : OpalRegisterRemoved;
   }
-  PTRACE(4, "OpalC\tOnRegistrationStatus " << aor << ", status=" << message->m_param.m_registrationStatus.m_status);
+  PTRACE(4, "OpalC\tOnRegistrationStatus " << status.m_addressofRecord << ", status=" << message->m_param.m_registrationStatus.m_status);
   m_manager.PostMessage(message);
 }
 
+
+void SIPEndPoint_C::OnSubscriptionStatus(const PString & eventPackage,
+                                         const SIPURL & uri,
+                                         bool wasSubscribing,
+                                         bool reSubscribing,
+                                         SIP_PDU::StatusCodes reason)
+{
+  if (reason == SIP_PDU::Successful_OK && !reSubscribing &&
+      eventPackage == SIPSubscribe::GetEventPackageName(SIPSubscribe::MessageSummary)) {
+    OpalMessageBuffer message(OpalIndMessageWaiting);
+    SET_MESSAGE_STRING(message, m_param.m_messageWaiting.m_party, uri.AsString());
+    SET_MESSAGE_STRING(message, m_param.m_messageWaiting.m_extraInfo, wasSubscribing ? "SUBSCRIBED" : "UNSUBSCRIBED");
+    PTRACE(4, "OpalC API\tOnSubscriptionStatus: party=\"" << message->m_param.m_messageWaiting.m_party
+                                            << "\" info=" << message->m_param.m_messageWaiting.m_extraInfo);
+    m_manager.PostMessage(message);
+  }
+
+  SIPEndPoint::OnSubscriptionStatus(eventPackage, uri, wasSubscribing, reSubscribing, reason);
+}
+
+
 #endif
 
 ///////////////////////////////////////
@@ -668,7 +721,7 @@ bool OpalManager_C::Initialise(const PCaselessString & options)
   }
 #endif
 
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
   if (pcPos != P_MAX_INDEX) {
     pcssEP = new OpalPCSSEndPoint_C(*this);
     AddRouteEntry("pc:.*=" + defProto + ":<da>");
@@ -768,7 +821,7 @@ OpalMessage * OpalManager_C::SendMessage(const OpalMessage * message)
 
 void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuffer & response)
 {
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
   if (pcssEP != NULL) {
     SET_MESSAGE_STRING(response, m_param.m_general.m_audioRecordDevice, pcssEP->GetSoundChannelRecordDevice());
     if (!IsNullString(command.m_param.m_general.m_audioRecordDevice))
@@ -778,7 +831,7 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
     if (!IsNullString(command.m_param.m_general.m_audioPlayerDevice))
       pcssEP->SetSoundChannelPlayDevice(command.m_param.m_general.m_audioPlayerDevice);
   }
-#endif // P_AUDIO
+#endif // OPAL_PTLIB_AUDIO
 
 #if OPAL_VIDEO
   PVideoDevice::OpenArgs video = GetVideoInputDevice();
@@ -847,7 +900,7 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
       response.SetError("Could not set STUN server address.");
       return;
     }
-    if (GetSTUN()->GetNatType() == PSTUNClient::BlockedNat)
+    if (GetSTUNClient()->GetNatType() == PSTUNClient::BlockedNat)
       response.SetError("STUN indicates Blocked NAT.");
   }
 
@@ -883,7 +936,7 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
     return;
 
   OpalSilenceDetector::Params silenceDetectParams = GetSilenceDetectParams();
-  response->m_param.m_general.m_silenceDetectMode = (OpalSilenceDetectModes)(silenceDetectParams.m_mode+1);
+  response->m_param.m_general.m_silenceDetectMode = (OpalSilenceDetectMode)(silenceDetectParams.m_mode+1);
   if (command.m_param.m_general.m_silenceDetectMode != 0)
     silenceDetectParams.m_mode = (OpalSilenceDetector::Mode)(command.m_param.m_general.m_silenceDetectMode-1);
   response->m_param.m_general.m_silenceThreshold = silenceDetectParams.m_threshold;
@@ -909,7 +962,7 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
   if (m_apiVersion < 3)
     return;
 
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
   if (pcssEP != NULL) {
     response->m_param.m_general.m_audioBuffers = pcssEP->GetSoundChannelBufferDepth();
     if (command.m_param.m_general.m_audioBuffers != 0)
@@ -946,27 +999,35 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
 
 void FillOpalProductInfo(const OpalMessage & command, OpalMessageBuffer & response, OpalProductInfo & info)
 {
-  SET_MESSAGE_STRING(response, m_param.m_protocol.m_vendor,  info.vendor);
-  SET_MESSAGE_STRING(response, m_param.m_protocol.m_name,    info.name);
-  SET_MESSAGE_STRING(response, m_param.m_protocol.m_version, info.version);
+  SET_MESSAGE_STRING(response, m_param.m_protocol.m_product.m_vendor,  info.vendor);
+  SET_MESSAGE_STRING(response, m_param.m_protocol.m_product.m_name,    BuildProductName(info));
+  SET_MESSAGE_STRING(response, m_param.m_protocol.m_product.m_version, info.version);
 
-  response->m_param.m_protocol.m_t35CountryCode   = info.t35CountryCode;
-  response->m_param.m_protocol.m_t35Extension     = info.t35Extension;
-  response->m_param.m_protocol.m_manufacturerCode = info.manufacturerCode;
+  response->m_param.m_protocol.m_product.m_t35CountryCode   = info.t35CountryCode;
+  response->m_param.m_protocol.m_product.m_t35Extension     = info.t35Extension;
+  response->m_param.m_protocol.m_product.m_manufacturerCode = info.manufacturerCode;
 
-  if (!IsNullString(command.m_param.m_protocol.m_vendor))
-    info.vendor = command.m_param.m_protocol.m_vendor;
+  if (command.m_param.m_protocol.m_product.m_vendor != NULL)
+    info.vendor = command.m_param.m_protocol.m_product.m_vendor;
 
-  if (!IsNullString(command.m_param.m_protocol.m_name))
-    info.name = command.m_param.m_protocol.m_name;
+  if (command.m_param.m_protocol.m_product.m_name != NULL) {
+    PString str = command.m_param.m_protocol.m_product.m_name;
+    PINDEX paren = str.Find('(');
+    if (paren == P_MAX_INDEX)
+      info.name = str;
+    else {
+      info.name = str.Left(paren).Trim();
+      info.comments = str.Mid(paren);
+    }
+  }
 
-  if (!IsNullString(command.m_param.m_protocol.m_version))
-    info.version = command.m_param.m_protocol.m_version;
+  if (command.m_param.m_protocol.m_product.m_version != NULL)
+    info.version = command.m_param.m_protocol.m_product.m_version;
 
-  if (command.m_param.m_protocol.m_t35CountryCode != 0 && command.m_param.m_protocol.m_manufacturerCode != 0) {
-    info.t35CountryCode   = (BYTE)command.m_param.m_protocol.m_t35CountryCode;
-    info.t35Extension     = (BYTE)command.m_param.m_protocol.m_t35Extension;
-    info.manufacturerCode = (WORD)command.m_param.m_protocol.m_manufacturerCode;
+  if (command.m_param.m_protocol.m_product.m_t35CountryCode != 0 && command.m_param.m_protocol.m_product.m_manufacturerCode != 0) {
+    info.t35CountryCode   = (BYTE)command.m_param.m_protocol.m_product.m_t35CountryCode;
+    info.t35Extension     = (BYTE)command.m_param.m_protocol.m_product.m_t35Extension;
+    info.manufacturerCode = (WORD)command.m_param.m_protocol.m_product.m_manufacturerCode;
   }
 }
 
@@ -1138,7 +1199,7 @@ void OpalManager_C::HandleSetUpCall(const OpalMessage & command, OpalMessageBuff
 
   PString partyA = command.m_param.m_callSetUp.m_partyA;
   if (partyA.IsEmpty()) {
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
     if (pcssEP != NULL)
       partyA = "pc:*";
     else
@@ -1168,7 +1229,7 @@ void OpalManager_C::HandleAnswerCall(const OpalMessage & command, OpalMessageBuf
   }
 
   if (
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
       pcssEP == NULL &&
 #endif
       localEP == NULL) {
@@ -1176,7 +1237,7 @@ void OpalManager_C::HandleAnswerCall(const OpalMessage & command, OpalMessageBuf
     return;
   }
 
-#if P_AUDIO
+#if OPAL_PTLIB_AUDIO
   if (pcssEP != NULL && pcssEP->AcceptIncomingConnection(command.m_param.m_callToken))
     return;
 #endif
@@ -1535,7 +1596,9 @@ void OpalManager_C::OnMWIReceived(const PString & party, MessageWaitingType type
   if ((size_t)type < sizeof(TypeNames)/sizeof(TypeNames[0]))
     SET_MESSAGE_STRING(message, m_param.m_messageWaiting.m_type, TypeNames[type]);
   SET_MESSAGE_STRING(message, m_param.m_messageWaiting.m_extraInfo, extraInfo);
-  PTRACE(4, "OpalC API\tOnMWIReceived: party=\"" << message->m_param.m_messageWaiting.m_party << '"');
+  PTRACE(4, "OpalC API\tOnMWIReceived: party=\"" << message->m_param.m_messageWaiting.m_party
+                                   << "\" type=" << message->m_param.m_messageWaiting.m_type
+                                   << "\" info=" << message->m_param.m_messageWaiting.m_extraInfo);
   PostMessage(message);
 
   OpalManager::OnMWIReceived(party, type, extraInfo);
