@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20800 $
+ * $Revision: 21014 $
  * $Author: rjongbloed $
- * $Date: 2008-09-01 04:01:44 +0200 (lun 01 sep 2008) $
+ * $Date: 2008-09-16 12:36:29 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -35,8 +35,10 @@
 #pragma implementation "patch.h"
 #endif
 
-#include <opal/patch.h>
 
+#include <opal/buildopts.h>
+
+#include <opal/patch.h>
 #include <opal/mediastrm.h>
 #include <opal/transcoders.h>
 
@@ -51,6 +53,7 @@ OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
 {
   src.SetPatch(this);
   patchThread = NULL;
+  PTRACE(5, "Patch\tCreated media patch " << this);
 }
 
 
@@ -63,7 +66,7 @@ OpalMediaPatch::~OpalMediaPatch()
     delete patchThread;
     patchThread = NULL;
   }
-  PTRACE(4, "Patch\tMedia patch thread " << *this << " destroyed.");
+  PTRACE(5, "Patch\tDestroyed media patch " << this);
 }
 
 
@@ -226,6 +229,7 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
     if (s->stream == stream) {
       sinks.erase(s);
+      PTRACE(5, "Patch\tRemoved media stream sink " << *stream);
       break;
     }
   }
@@ -283,7 +287,7 @@ void OpalMediaPatch::UnLockSinkTranscoder() const
 }
 
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
 void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics) const
 {
   inUse.StartRead();
@@ -386,7 +390,7 @@ PBoolean OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat,
   PReadWaitAndSignal mutex(inUse);
 
   if (fromSink)
-    return source.UpdateMediaFormat(mediaFormat);
+    return source.UpdateMediaFormat(mediaFormat, true);
 
   PBoolean atLeastOne = PFalse;
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
@@ -499,7 +503,7 @@ bool OpalMediaPatch::Sink::UpdateMediaFormat(const OpalMediaFormat & mediaFormat
   bool ok;
 
   if (primaryCodec == NULL) {
-    ok = stream->UpdateMediaFormat(mediaFormat);
+    ok = stream->UpdateMediaFormat(mediaFormat, true);
 #if OPAL_VIDEO
     SetRateControlParameters(stream->GetMediaFormat());
 #endif
