@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20627 $
+ * $Revision: 21004 $
  * $Author: rjongbloed $
- * $Date: 2008-07-28 03:58:31 +0200 (lun 28 jui 2008) $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -143,7 +143,7 @@ RTP_Session * OpalRTPConnection::CreateSession(const OpalTransport & transport,
 
   PIPSocket::Address remoteAddress;
   transport.GetRemoteAddress().GetIpAddress(remoteAddress);
-  PSTUNClient * stun = manager.GetSTUN(remoteAddress);
+  PNatMethod * natMethod = manager.GetNatMethod(remoteAddress);
 
   OpalMediaType mediaType = OpalMediaTypeDefinition::GetMediaTypeForSessionId(sessionID);
   OpalMediaTypeDefinition * def = mediaType.GetDefinition();
@@ -182,7 +182,7 @@ RTP_Session * OpalRTPConnection::CreateSession(const OpalTransport & transport,
 
   WORD firstPort = manager.GetRtpIpPortPair();
   WORD nextPort = firstPort;
-  while (!rtpSession->Open(localAddress, nextPort, nextPort, manager.GetRtpIpTypeofService(), stun, rtpqos)) {
+  while (!rtpSession->Open(localAddress, nextPort, nextPort, manager.GetRtpIpTypeofService(), natMethod, rtpqos)) {
     nextPort = manager.GetRtpIpPortPair();
     if (nextPort == firstPort) {
       PTRACE(1, "RTPCon\tNo ports available for RTP session " << sessionID << " for " << *this);
@@ -246,7 +246,7 @@ void OpalRTPConnection::OnMediaCommand(OpalMediaCommand & command, INT /*extra*/
     RTP_Session * session = m_rtpSessions.GetSession(OpalMediaFormat::DefaultVideoSessionID);
     if (session != NULL)
       session->SendIntraFrameRequest();
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     m_VideoUpdateRequestsSent++;
 #endif
   }
@@ -338,7 +338,7 @@ void OpalRTPConnection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & p
   OpalConnection::OnPatchMediaStream(isSource, patch);
   if (patch.GetSource().GetMediaFormat().GetMediaType() == OpalMediaType::Audio()) {
     AttachRFC2833HandlerToPatch(isSource, patch);
-#if P_DTMF
+#if OPAL_PTLIB_DTMF
     if (detectInBandDTMF && isSource) {
       patch.AddFilter(PCREATE_NOTIFIER(OnUserInputInBandDTMF), OPAL_PCM16);
     }
@@ -375,7 +375,8 @@ OpalMediaSession::OpalMediaSession(const OpalMediaType & _mediaType)
 
 
 OpalMediaSession::OpalMediaSession(const OpalMediaSession & _obj)
-  : mediaType(_obj.mediaType)
+  : PObject(_obj)
+  , mediaType(_obj.mediaType)
   , autoStartReceive(_obj.autoStartReceive)
   , autoStartTransmit(_obj.autoStartTransmit)
   , sessionId(_obj.sessionId)
