@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20555 $
- * $Author: csoutheren $
- * $Date: 2008-07-09 06:34:59 +0200 (mer 09 jui 2008) $
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -37,6 +37,8 @@
 #pragma implementation "jitter.h"
 #endif
 
+#include <opal/buildopts.h>
+
 #include <rtp/jitter.h>
 
 /*Number of consecutive attempts to add a packet to the jitter buffer while
