@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20672 $
+ * $Revision: 21035 $
  * $Author: rjongbloed $
- * $Date: 2008-08-15 10:32:41 +0200 (ven 15 aoû 2008) $
+ * $Date: 2008-09-17 07:57:58 +0000 (Wed, 17 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -37,6 +37,8 @@
 #pragma implementation "rtp.h"
 #endif
 
+#include <opal/buildopts.h>
+
 #include <rtp/rtp.h>
 
 #include <rtp/jitter.h>
@@ -426,7 +428,7 @@ void RTP_ControlFrame::ReceiverReport::SetLostPackets(unsigned packets)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
 
 OpalMediaStatistics::OpalMediaStatistics()
   : m_totalBytes(0)
@@ -462,6 +464,10 @@ void RTP_UserData::OnRxStatistics(const RTP_Session & /*session*/) const
 {
 }
 
+void RTP_UserData::OnClearCall(const RTP_Session & /*session*/)
+{
+}
+
 #if OPAL_VIDEO
 void RTP_UserData::OnRxIntraFrameRequest(const RTP_Session & /*session*/) const
 {
@@ -475,26 +481,21 @@ void RTP_UserData::OnTxIntraFrameRequest(const RTP_Session & /*session*/) const
 
 /////////////////////////////////////////////////////////////////////////////
 
-RTP_Session::RTP_Session(
-                         const PString & encoding,
-#if OPAL_RTP_AGGREGATE
-                         PHandleAggregator * _aggregator, 
-#endif
-                         unsigned id, RTP_UserData * data, PBoolean autoDelete)
+RTP_Session::RTP_Session(const Params & params)
 : canonicalName(PProcess::Current().GetUserName()),
   toolName(PProcess::Current().GetName()),
   reportTimeInterval(0, 12),  // Seconds
   reportTimer(reportTimeInterval)
 #if OPAL_RTP_AGGREGATE
-  , aggregator(_aggregator)
+  , aggregator(params.aggregator)
 #endif
 {
-  PAssert(id > 0 && id < 256, PInvalidParameter);
-  sessionID = (BYTE)id;
-  isAudio = sessionID == 1;  // MAJOR ASSUMPTION!!!
+  PAssert(params.id > 0 && params.id < 256, PInvalidParameter);
+  sessionID = (BYTE)params.id;
+  isAudio = params.isAudio;
 
-  userData = data;
-  autoDeleteUserData = autoDelete;
+  userData = params.userData;
+  autoDeleteUserData = params.autoDelete;
   jitter = NULL;
 
   ignoreOutOfOrderPackets = true;
@@ -554,7 +555,7 @@ RTP_Session::RTP_Session(
   lastSentTimestamp = 0;  // should be calculated, but we'll settle for initialising it
 
   m_encodingHandler = NULL;
-  SetEncoding(encoding);
+  SetEncoding(params.encoding);
 }
 
 RTP_Session::~RTP_Session()
@@ -581,6 +582,7 @@ RTP_Session::~RTP_Session()
   delete jitter;
   if (autoDeleteUserData)
     delete userData;
+  delete m_encodingHandler;
 }
 
 void RTP_Session::SendBYE()
@@ -1170,7 +1172,7 @@ PBoolean RTP_Session::SendReport()
 }
 
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
 void RTP_Session::GetStatistics(OpalMediaStatistics & statistics, bool receiver) const
 {
   statistics.m_totalBytes        = receiver ? GetOctetsReceived()     : GetOctetsSent();
@@ -1471,28 +1473,23 @@ static void SetMinBufferSize(PUDPSocket & sock, int buftype)
     if (sz >= UDP_BUFFER_SIZE)
       return;
   }
+  else {
+    PTRACE(1, "RTP_UDP\tGetOption(" << buftype << ") failed: " << sock.GetErrorText());
+  }
 
   if (!sock.SetOption(buftype, UDP_BUFFER_SIZE)) {
     PTRACE(1, "RTP_UDP\tSetOption(" << buftype << ") failed: " << sock.GetErrorText());
   }
+
+  PTRACE_IF(1, !sock.GetOption(buftype, sz) && sz < UDP_BUFFER_SIZE, "RTP_UDP\tSetOption(" << buftype << ") failed, even though it said it succeeded!");
 }
 
 
-RTP_UDP::RTP_UDP(
-                 const PString & _format,
-#if OPAL_RTP_AGGREGATE
-                 PHandleAggregator * _aggregator, 
-#endif
-                 unsigned id, PBoolean _remoteIsNAT)
-  : RTP_Session(
-    _format,
-#if OPAL_RTP_AGGREGATE
-  _aggregator, 
-#endif
-  id),
+RTP_UDP::RTP_UDP(const Params & params)
+  : RTP_Session(params),
     remoteAddress(0),
     remoteTransmitAddress(0),
-    remoteIsNAT(_remoteIsNAT)
+    remoteIsNAT(params.remoteIsNAT)
 {
   PTRACE(4, "RTP_UDP\tSession " << sessionID << ", created with NAT flag set to " << remoteIsNAT);
   remoteDataPort    = 0;
@@ -1503,6 +1500,7 @@ RTP_UDP::RTP_UDP(
   controlSocket     = NULL;
   appliedQOS        = false;
   localHasNAT       = false;
+  badTransmitCounter = 0;
 }
 
 
@@ -1846,29 +1844,19 @@ int RTP_UDP::WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket, con
 
 int RTP_UDP::Internal_WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket, const PTimeInterval & timeout)
 {
-  if (first && (sessionID == 1)) {
-    BYTE buffer[1500];
-    PINDEX count = 0;
-    do {
-      switch (PSocket::Select(dataSocket, controlSocket, PTimeInterval(0))) {
-        case -2:
-          controlSocket.Read(buffer, sizeof(buffer));
-          ++count;
-          break;
+  if (first && isAudio) {
+    PTimeInterval oldTimeout = dataSocket.GetReadTimeout();
+    dataSocket.SetReadTimeout(0);
 
-        case -3:
-          controlSocket.Read(buffer, sizeof(buffer));
-          ++count;
-        case -1:
-          dataSocket.Read(buffer, sizeof(buffer));
-          ++count;
-          break;
-        case 0:
-          first = false;
-          break;
-      }
-    } while (first);
-    PTRACE_IF(2, count > 0, "RTP_UDP\tSession " << sessionID << ", swallowed " << count << " RTP packets on startup");
+    BYTE buffer[2000];
+    PINDEX count = 0;
+    while (dataSocket.Read(buffer, sizeof(buffer)))
+      ++count;
+
+    PTRACE_IF(2, count > 0, "RTP_UDP\tSession " << sessionID << ", flushed " << count << " RTP data packets on startup");
+
+    dataSocket.SetReadTimeout(oldTimeout);
+    first = false;
   }
 
   return PSocket::Select(dataSocket, controlSocket, timeout);
@@ -1918,15 +1906,23 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(BYTE * framePtr,
     if (remoteAddress.IsValid() && !appliedQOS) 
       ApplyQOS(remoteAddress);
 
+    badTransmitCounter = 0;
+
     return RTP_Session::e_ProcessPacket;
   }
 
   switch (socket.GetErrorNumber()) {
     case ECONNRESET :
     case ECONNREFUSED :
-      PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName
-             << " port on remote not ready.");
-      return RTP_Session::e_IgnorePacket;
+      if (++badTransmitCounter < 25) { 
+        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " port on remote not ready.");
+        return RTP_Session::e_IgnorePacket;
+      }
+      else {
+        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " too many transmit fails - killing call");
+        userData->OnClearCall(*this);
+        return RTP_Session::e_AbortTransport;
+      }
 
     case EMSGSIZE :
       PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName
@@ -2245,17 +2241,8 @@ int RTP_Encoding::WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket
 
 /////////////////////////////////////////////////////////////////////////////
 
-SecureRTP_UDP::SecureRTP_UDP(const PString & encoding,
-#if OPAL_RTP_AGGREGATE
-
-                             PHandleAggregator * _aggregator, 
-#endif
-                             unsigned id, PBoolean remoteIsNAT)
-  : RTP_UDP(encoding,
-#if OPAL_RTP_AGGREGATE
-            _aggregator, 
-#endif
-            id, remoteIsNAT)
+SecureRTP_UDP::SecureRTP_UDP(const Params & params)
+  : RTP_UDP(params)
 {
   securityParms = NULL;
 }
