@@ -23,14 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20798 $
- * $Author: csoutheren $
- * $Date: 2008-08-31 15:09:49 +0200 (dim 31 aoû 2008) $
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
-
 #include <opal/buildopts.h>
+
 #if OPAL_SIP
 
 #ifdef __GNUC__
@@ -100,7 +100,7 @@ static OpalTransportAddress ParseConnectAddress(const PStringArray & tokens, PIN
     if (tokens[offset] *= "IN") {
       if (
         (tokens[offset+1] *= "IP4")
-#if P_HAS_IPV6
+#if OPAL_PTLIB_IPV6
         || (tokens[offset+1] *= "IP6")
 #endif
         ) {
@@ -189,7 +189,6 @@ SDPMediaFormat::SDPMediaFormat(const OpalMediaFormat & fmt,
     parameters = PString(PString::Unsigned, fmt.GetOptionInteger(OpalAudioFormat::ChannelsOption()));
 }
 
-
 void SDPMediaFormat::SetFMTP(const PString & str)
 {
   if (str.IsEmpty())
@@ -213,7 +212,7 @@ void SDPMediaFormat::SetFMTP(const PString & str)
   if (GetMediaFormat().IsEmpty()) // Use GetMediaFormat() to force creation of member
     return;
 
-  mediaFormat.SetOptionString("RawFMTP", str); // Save the 'fmtp=' line so it is available at the application level.
+  mediaFormat.AddOption(new OpalMediaOptionString("RawFMTP", false, str), PTrue); // Save the 'fmtp=' line so it is available at the application level.
 
   // See if standard format OPT=VAL;OPT=VAL
   if (str.FindOneOf(";=") == P_MAX_INDEX) {
@@ -255,8 +254,14 @@ void SDPMediaFormat::SetFMTP(const PString & str)
     }
     if (option != NULL) {
       PString value = str(sep2pos+1, sep1next-1);
-      if (value.Trim().IsEmpty())
-        value = "1"; // Assume it is a boolean
+      if (dynamic_cast< OpalMediaOptionOctets * >(option) != NULL) {
+        if (str.GetLength() % 2 != 0)
+          value = value.Trim();
+      } else {
+        value = value.Trim();
+        if (value.IsEmpty())
+          value = "1"; // Assume it is a boolean
+      }
       if (!option->FromString(value)) {
         PTRACE(2, "SDP\tCould not set FMTP parameter \"" << key << "\" to value \"" << value << '"');
       }
