@@ -23,14 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20784 $
- * $Author: csoutheren $
- * $Date: 2008-08-31 06:30:01 +0200 (dim 31 aoû 2008) $
+ * $Revision: 21029 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 23:58:39 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
-
 #include <opal/buildopts.h>
+
 #if OPAL_SIP
 
 #ifdef __GNUC__
@@ -313,9 +313,12 @@ bool SIPConnection::TransferConnection(const PString & remoteParty)
   if (referTransaction != NULL) 
     return false;
 
+  SIPURL localPartyURL = endpoint.GetRegisteredPartyName(remoteParty);
+  localPartyURL.Sanitise(SIPURL::RequestURI);
+
   PSafePtr<OpalCall> call = endpoint.GetManager().FindCallWithLock(remoteParty, PSafeReadOnly);
   if (call == NULL) {
-    referTransaction = new SIPRefer(*this, *transport, remoteParty, GetLocalPartyURL());
+    referTransaction = new SIPRefer(*this, *transport, remoteParty, localPartyURL);
     return referTransaction->Start();
   }
 
@@ -327,7 +330,7 @@ bool SIPConnection::TransferConnection(const PString & remoteParty)
               << "?Replaces=" << sip->GetToken()
               << "%3Bto-tag%3D"   << SIPMIMEInfo::ExtractFieldParameter(sip->GetDialogFrom(), "tag") // "to/from" is from the other sides perspective
               << "%3Bfrom-tag%3D" << SIPMIMEInfo::ExtractFieldParameter(sip->GetDialogTo(), "tag");
-      referTransaction = new SIPRefer(*this, *transport, referTo, GetLocalPartyURL());
+      referTransaction = new SIPRefer(*this, *transport, referTo, localPartyURL);
       referTransaction->GetMIME().SetAt("Refer-Sub", "false"); // Use RFC4488 to indicate we are NOT doing NOTIFYs
       return referTransaction->Start();
     }
@@ -975,7 +978,7 @@ OpalMediaStreamPtr SIPConnection::OpenMediaStream(const OpalMediaFormat & mediaF
 
   if (GetPhase() == EstablishedPhase && needReINVITE) {
     PTRACE(3, "SIP\tStarting re-INVITE to open channel.");
-    SIPTransaction * invite = new SIPInvite(*this, *transport, m_rtpSessions);
+    SIPTransaction * invite = new SIPInvite(*this, *transport, &m_rtpSessions);
     invite->Start();
   }
 
@@ -989,7 +992,7 @@ bool SIPConnection::CloseMediaStream(OpalMediaStream & stream)
 
   if (GetPhase() == EstablishedPhase && needReINVITE) {
     PTRACE(3, "SIP\tStarting re-INVITE to close channel.");
-    SIPTransaction * invite = new SIPInvite(*this, *transport, m_rtpSessions);
+    SIPTransaction * invite = new SIPInvite(*this, *transport, &m_rtpSessions);
     invite->Start();
   }
 
@@ -1003,7 +1006,7 @@ PBoolean SIPConnection::WriteINVITE(OpalTransport & transport, void * param)
 
   connection.AdjustOutgoingINVITE();
   connection.needReINVITE = false;
-  SIPTransaction * invite = new SIPInvite(connection, transport);
+  SIPTransaction * invite = new SIPInvite(connection, transport, NULL);
   
   // It may happen that constructing the INVITE causes the connection
   // to be released (e.g. there are no UDP ports available for the RTP sessions)
@@ -1071,6 +1074,14 @@ PString SIPConnection::GetDestinationAddress()
 }
 
 
+PString SIPConnection::GetCalledPartyURL()
+{
+  SIPURL calledParty = originalInvite != NULL ? originalInvite->GetMIME().GetTo() : m_dialogTo;
+  calledParty.Sanitise(SIPURL::ToURI);
+  return calledParty.AsString();
+}
+
+
 bool SIPConnection::HoldConnection()
 {
   if (transport == NULL)
@@ -1083,7 +1094,7 @@ bool SIPConnection::HoldConnection()
 
   m_holdToRemote = eHoldInProgress;
 
-  SIPTransaction * invite = new SIPInvite(*this, *transport, m_rtpSessions);
+  SIPTransaction * invite = new SIPInvite(*this, *transport, &m_rtpSessions);
   if (invite->Start())
     return true;
 
@@ -1112,7 +1123,7 @@ bool SIPConnection::RetrieveConnection()
 
   PTRACE(3, "SIP\tStarting re-INVITE to retrieve connection from hold");
 
-  SIPTransaction * invite = new SIPInvite(*this, *transport, m_rtpSessions);
+  SIPTransaction * invite = new SIPInvite(*this, *transport, &m_rtpSessions);
   if (invite->Start())
     return true;
 
@@ -1162,13 +1173,6 @@ PString SIPConnection::GetPrefixName() const
 }
 
 
-PString SIPConnection::GetLocalPartyURL() const
-{
-  SIPURL url(localPartyName, transport->GetLocalAddress());
-  return url.AsString();
-}
-
-
 PString SIPConnection::GetRemotePartyURL() const
 {
   SIPURL url = GetRemotePartyAddress();
@@ -1910,7 +1914,7 @@ PBoolean SIPConnection::OnReceivedAuthenticationRequired(SIPTransaction & transa
   transport->SetInterface(transaction.GetInterface());
   OpalRTPSessionManager & origRtpSessions = ((SIPInvite &)transaction).GetSessionManager();
   (((SIPInvite &)transaction).GetSessionManager()).SetCleanup(false);
-  SIPTransaction * invite = new SIPInvite(*this, *transport, origRtpSessions);
+  SIPTransaction * invite = new SIPInvite(*this, *transport, &origRtpSessions);
 
   // Section 8.1.3.5 of RFC3261 tells that the authenticated
   // request SHOULD have the same value of the Call-ID, To and From.
@@ -2013,6 +2017,7 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
 
     return false;
   }
+  PTRACE(4, "SIP\tProcessing received SDP media description for " << mediaType);
 
   // see if the remote supports this media
   OpalMediaFormatList mediaFormatList = mediaDescription->GetMediaFormats();
@@ -2075,6 +2080,7 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
   if ((otherSidesDir&SDPMediaDescription::RecvOnly) != 0 && otherParty != NULL)
     ownerCall.OpenSourceMediaStreams(*otherParty, mediaType, rtpSessionId);
 
+  PTRACE_IF(3, otherSidesDir == SDPMediaDescription::Inactive, "SIP\tNo streams opened as " << mediaType << " inactive");
   return true;
 }
 
@@ -2091,6 +2097,9 @@ void SIPConnection::OnCreatingINVITE(SIP_PDU & request)
       mime.SetAt("Replaces", replaces);
     }
   }
+
+  if (!request.GetSDP() || request.GetSDP()->GetMediaDescriptions().GetSize () == 0)
+    Release(EndedByCapabilityExchange);
 }
 
 PBoolean SIPConnection::ForwardCall (const PString & fwdParty)
@@ -2123,6 +2132,7 @@ PBoolean SIPConnection::SendInviteResponse(SIP_PDU::StatusCodes code, const char
 
   SIP_PDU response(*originalInvite, code, contact, extra, sdp);
   response.GetMIME().SetProductInfo(endpoint.GetUserAgent(), GetProductInfo());
+  response.SetAllow(endpoint.GetAllowedMethods());
 
   if (response.GetStatusCode() >= 200) {
     ackPacket = response;
@@ -2298,7 +2308,7 @@ void SIPConnection::OnMediaCommand(OpalMediaCommand & command, INT extra)
     //infoTransaction->WaitForCompletion();
     //if (infoTransaction->IsFailed()) { }
     infoTransaction->Start();
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     m_VideoUpdateRequestsSent++;
 #endif
   }
@@ -2466,7 +2476,7 @@ void SIP_RTP_Session::OnRxStatistics(const RTP_Session & session) const
   connection.OnRTPStatistics(session);
 }
 
-#ifdef OPAL_VIDEO
+#if OPAL_VIDEO
 void SIP_RTP_Session::OnRxIntraFrameRequest(const RTP_Session & session) const
 {
   // We got an intra frame request control packet, alert the encoder.
@@ -2490,6 +2500,11 @@ void SIP_RTP_Session::OnTxIntraFrameRequest(const RTP_Session & /*session*/) con
 
 #endif // OPAL_VIDEO
 
+void SIP_RTP_Session::OnClearCall(const RTP_Session & /*session*/)
+{
+  ((SIPConnection &)connection).ClearCall();
+}
+
 void SIPConnection::OnSessionTimeout(PTimer &, INT)
 {
   //SIPTransaction * invite = new SIPInvite(*this, *transport, rtpSessions);  
