@@ -23,15 +23,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20790 $
+ * $Revision: 21062 $
  * $Author: rjongbloed $
- * $Date: 2008-08-31 08:15:19 +0200 (dim 31 aoû 2008) $
+ * $Date: 2008-09-18 11:41:40 +0000 (Thu, 18 Sep 2008) $
  */
 
 #include <ptlib.h>
-
 #include <opal/buildopts.h>
-#ifdef OPAL_SIP
+
+#if OPAL_SIP
 
 #ifdef __GNUC__
 #pragma implementation "sipep.h"
@@ -53,16 +53,23 @@
 ////////////////////////////////////////////////////////////////////////////
 
 SIPEndPoint::SIPEndPoint(OpalManager & mgr)
-: OpalRTPEndPoint(mgr, "sip", CanTerminateCall),
-    retryTimeoutMin(500),            // 0.5 seconds
-    retryTimeoutMax(0, 4),           // 4 seconds
-    nonInviteTimeout(0, 16),         // 16 seconds
-    pduCleanUpTimeout(0, 5),         // 5 seconds
-    inviteTimeout(0, 32),            // 32 seconds
-    ackTimeout(0, 32),               // 32 seconds
-    registrarTimeToLive(0, 0, 0, 1), // 1 hour
-    notifierTimeToLive(0, 0, 0, 1),  // 1 hour
-    natBindingTimeout(0, 0, 1)       // 1 minute
+  : OpalRTPEndPoint(mgr, "sip", CanTerminateCall)
+  , retryTimeoutMin(500)             // 0.5 seconds
+  , retryTimeoutMax(0, 4)            // 4 seconds
+  , nonInviteTimeout(0, 16)          // 16 seconds
+  , pduCleanUpTimeout(0, 5)          // 5 seconds
+  , inviteTimeout(0, 32)             // 32 seconds
+  , ackTimeout(0, 32)                // 32 seconds
+  , registrarTimeToLive(0, 0, 0, 1)  // 1 hour
+  , notifierTimeToLive(0, 0, 0, 1)   // 1 hour
+  , natBindingTimeout(0, 0, 1)       // 1 minute
+#ifdef _MSC_VER
+#pragma warning(disable:4355)
+#endif
+  , m_interfaceMonitor(*this)
+#ifdef _MSC_VER
+#pragma warning(default:4355)
+#endif
 {
   defaultSignalPort = 5060;
   mimeForm = PFalse;
@@ -81,7 +88,7 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr)
   GetOpalCiscoNSE();
 #endif
 
-#ifdef P_SSL
+#if OPAL_PTLIB_SSL
   manager.AttachEndPoint(this, "sips");
 #endif
 
@@ -105,11 +112,14 @@ void SIPEndPoint::ShutDown()
   bool shuttingDown = true;
   while (shuttingDown) {
     shuttingDown = false;
-    for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetAt(0, PSafeReference); handler != NULL; ++handler) {
+    PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReference);
+    while (handler != NULL) {
       if (handler->ShutDown())
         activeSIPHandlers.Remove(handler++);
-      else
+      else {
         shuttingDown = true;
+        ++handler;
+      }
     }
     PThread::Sleep(100);
   }
@@ -129,7 +139,7 @@ void SIPEndPoint::ShutDown()
 PString SIPEndPoint::GetDefaultTransport() const 
 {  
   return "udp$,tcp$"
-#ifdef P_SSL
+#if OPAL_PTLIB_SSL
          ",tcps$:5061"
 #endif
     ; 
@@ -172,7 +182,10 @@ void SIPEndPoint::NATBindingRefresh(PTimer &, INT)
     for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
 
       OpalTransport * transport = NULL;
-      if (handler->GetState () != SIPHandler::Subscribed || (transport = handler->GetTransport()) == NULL || transport->IsReliable() || GetManager().GetSTUN(transport->GetRemoteAddress().GetHostName()) == NULL)
+      if (handler->GetState () != SIPHandler::Subscribed ||
+           (transport = handler->GetTransport()) == NULL ||
+           transport->IsReliable() ||
+           GetManager().GetNatMethod(transport->GetRemoteAddress().GetHostName()) == NULL)
         continue;
 
       switch (natMethod) {
@@ -255,9 +268,13 @@ void SIPEndPoint::HandlePDU(OpalTransport & transport)
     if (OnReceivedPDU(transport, pdu)) 
       return;
   }
-  else if (transport.good()) {
-    PTRACE(2, "SIP\tMalformed request received on " << transport);
-    SendResponse(SIP_PDU::Failure_BadRequest, transport, *pdu);
+  else {
+    PTRACE_IF(1, transport.GetErrorCode(PChannel::LastReadError) != PChannel::NoError,
+              "SIP\tPDU Read failed: " << transport.GetErrorText(PChannel::LastReadError));
+    if (transport.good()) {
+      PTRACE(2, "SIP\tMalformed request received on " << transport);
+      SendResponse(SIP_PDU::Failure_BadRequest, transport, *pdu);
+    }
   }
 
   delete pdu;
@@ -266,7 +283,7 @@ void SIPEndPoint::HandlePDU(OpalTransport & transport)
 
 static PString TranslateENUM(const PString & remoteParty)
 {
-#if P_DNS
+#if OPAL_PTLIB_DNS
   // if there is no '@', and then attempt to use ENUM
   if (remoteParty.Find('@') == P_MAX_INDEX) {
 
@@ -325,6 +342,15 @@ PBoolean SIPEndPoint::GarbageCollection()
   }
 
   transactions.DeleteObjectsToBeRemoved();
+
+  PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReference);
+  while (handler != NULL) {
+    if (handler->GetState() == SIPHandler::Unsubscribed && handler->ShutDown())
+      activeSIPHandlers.Remove(handler++);
+    else
+      ++handler;
+  }
+
   activeSIPHandlers.DeleteObjectsToBeRemoved();
 
   return OpalEndPoint::GarbageCollection();
@@ -447,50 +473,59 @@ bool SIPEndPoint::OnReceivedConnectionlessPDU(OpalTransport & transport, SIP_PDU
         if (transaction != NULL)
           transaction->OnReceivedResponse(*pdu);
       }
-      break;
+      return false;
 
     case SIP_PDU::Method_INVITE :
       return OnReceivedINVITE(transport, pdu);
 
     case SIP_PDU::Method_REGISTER :
-      return OnReceivedREGISTER(transport, *pdu);
+      if (OnReceivedREGISTER(transport, *pdu))
+        return false;
+      break;
 
     case SIP_PDU::Method_SUBSCRIBE :
-      return OnReceivedSUBSCRIBE(transport, *pdu);
+      if (OnReceivedSUBSCRIBE(transport, *pdu))
+        return false;
+      break;
 
     case SIP_PDU::Method_NOTIFY :
-       return OnReceivedNOTIFY(transport, *pdu);
+       if (OnReceivedNOTIFY(transport, *pdu))
+         return false;
+       break;
 
     case SIP_PDU::Method_MESSAGE :
-      OnReceivedMESSAGE(transport, *pdu);
-      SendResponse(SIP_PDU::Successful_OK, transport, *pdu);
+      if (OnReceivedMESSAGE(transport, *pdu))
+        return false;
       break;
    
     case SIP_PDU::Method_OPTIONS :
-      SendResponse(SIP_PDU::Successful_OK, transport, *pdu);
+      if (OnReceivedOPTIONS(transport, *pdu))
+        return false;
       break;
 
     case SIP_PDU::Method_ACK :
       // If we receive an ACK outside of the context of a connection, ignore it.
-      break;
+      SendResponse(SIP_PDU::Failure_TransactionDoesNotExist, transport, *pdu);
+      return false;
 
     default :
-      SendResponse(SIP_PDU::Failure_TransactionDoesNotExist, transport, *pdu);
+      break;
   }
 
+  SIP_PDU response(*pdu, SIP_PDU::Failure_MethodNotAllowed);
+  response.SetAllow(GetAllowedMethods()); // Required by spec
+  pdu->SendResponse(transport, response);
   return false;
 }
 
-PBoolean SIPEndPoint::OnReceivedREGISTER(OpalTransport & transport, SIP_PDU & pdu)
+PBoolean SIPEndPoint::OnReceivedREGISTER(OpalTransport & /*transport*/, SIP_PDU & /*pdu*/)
 {
-  SendResponse(SIP_PDU::Failure_MethodNotAllowed, transport, pdu);
-  return PFalse;
+  return false;
 }
 
-PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & pdu)
+PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & /*transport*/, SIP_PDU & /*pdu*/)
 {
-  SendResponse(SIP_PDU::Failure_MethodNotAllowed, transport, pdu);
-  return PFalse;
+  return false;
 }
 
 void SIPEndPoint::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response)
@@ -720,17 +755,16 @@ PBoolean SIPEndPoint::OnReceivedNOTIFY(OpalTransport & transport, SIP_PDU & pdu)
   if (handler == NULL) {
     PTRACE(3, "SIP\tCould not find a SUBSCRIBE corresponding to the NOTIFY");
     SendResponse(SIP_PDU::Failure_TransactionDoesNotExist, transport, pdu);
-    return PFalse;
+    return true;
   }
 
   PTRACE(3, "SIP\tFound a SUBSCRIBE corresponding to the NOTIFY");
   handler->OnReceivedNOTIFY(pdu);
-  return false;
+  return true;
 }
 
 
-void SIPEndPoint::OnReceivedMESSAGE(OpalTransport & /*transport*/, 
-            SIP_PDU & pdu)
+bool SIPEndPoint::OnReceivedMESSAGE(OpalTransport & transport, SIP_PDU & pdu)
 {
   PString from = pdu.GetMIME().GetFrom();
   PINDEX j = from.Find (';');
@@ -741,6 +775,21 @@ void SIPEndPoint::OnReceivedMESSAGE(OpalTransport & /*transport*/,
     from += '>';
 
   OnMessageReceived(from, pdu.GetEntityBody());
+  SendResponse(SIP_PDU::Successful_OK, transport, pdu);
+  return true;
+}
+
+
+bool SIPEndPoint::OnReceivedOPTIONS(OpalTransport & transport, SIP_PDU & pdu)
+{
+  SendResponse(SIP_PDU::Successful_OK, transport, pdu);
+  return true;
+}
+
+
+void SIPEndPoint::OnRegistrationStatus(const RegistrationStatus & status)
+{
+  OnRegistrationStatus(status.m_addressofRecord, status.m_wasRegistering, status.m_reRegistering, status.m_reason);
 }
 
 
@@ -792,6 +841,15 @@ PBoolean SIPEndPoint::IsSubscribed(const PString & eventPackage, const PString &
 }
 
 
+void SIPEndPoint::OnSubscriptionStatus(const PString & /*eventPackage*/,
+                                       const SIPURL & /*aor*/,
+                                       bool /*wasSubscribing*/,
+                                       bool /*reSubscribing*/,
+                                       SIP_PDU::StatusCodes /*reason*/)
+{
+}
+
+
 bool SIPEndPoint::Register(const PString & host,
                            const PString & user,
                            const PString & authName,
@@ -918,8 +976,9 @@ bool SIPEndPoint::Subscribe(const SIPSubscribe::Params & params)
   
   // If there is already a request with this URL and method, 
   // then update it with the new information
-  if (handler != NULL)
+  if (handler != NULL && handler->GetState() != SIPHandler::Unsubscribed) {
     handler->UpdateParameters(params);
+  }
   else {
     // Otherwise create a new request with this method type
     handler = new SIPSubscribeHandler(*this, params);
@@ -949,6 +1008,26 @@ bool SIPEndPoint::Unsubscribe(const PString & eventPackage, const PString & to)
 }
 
 
+bool SIPEndPoint::UnsubcribeAll(SIPSubscribe::PredefinedPackages eventPackage)
+{
+  return UnsubcribeAll(SIPSubscribe::GetEventPackageName(eventPackage));
+}
+
+
+bool SIPEndPoint::UnsubcribeAll(const PString & eventPackage)
+{
+  bool ok = true;
+
+  for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    if (handler->GetMethod() == SIP_PDU::Method_SUBSCRIBE && handler->GetEventPackage() == eventPackage) {
+      if (!handler->SendRequest(SIPHandler::Unsubscribing))
+        ok = false;
+    }
+  }
+
+  return ok;
+}
+
 
 PBoolean SIPEndPoint::Message (const PString & to, 
                                const PString & body)
@@ -1058,6 +1137,22 @@ PString SIPEndPoint::GetUserAgent() const
 }
 
 
+unsigned SIPEndPoint::GetAllowedMethods() const
+{
+  return (1<<SIP_PDU::Method_INVITE   )|
+         (1<<SIP_PDU::Method_ACK      )|
+         (1<<SIP_PDU::Method_CANCEL   )|
+         (1<<SIP_PDU::Method_BYE      )|
+         (1<<SIP_PDU::Method_OPTIONS  )|
+         (1<<SIP_PDU::Method_NOTIFY   )|
+         (1<<SIP_PDU::Method_REFER    )|
+         (1<<SIP_PDU::Method_MESSAGE  )|
+         (1<<SIP_PDU::Method_INFO     )|
+         (1<<SIP_PDU::Method_PING     )|
+         (1<<SIP_PDU::Method_SUBSCRIBE);
+}
+
+
 PBoolean SIPEndPoint::GetAuthentication(const PString & authRealm, PString & realm, PString & user, PString & password) 
 {
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(authRealm, PString::Empty(), PSafeReadOnly);
@@ -1228,6 +1323,34 @@ void SIPEndPoint::SIP_PDU_Thread::Main()
   }
 }
 
+
+SIPEndPoint::InterfaceMonitor::InterfaceMonitor(SIPEndPoint & ep)
+  : PInterfaceMonitorClient(30)  // Low priority, after the bundled socket has been done
+  , m_endpoint(ep)
+{
+}
+
+        
+void SIPEndPoint::InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry &)
+{
+  for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    if (handler->GetState() == SIPHandler::Unavailable)
+      handler->SendRequest(SIPHandler::Restoring);
+  }
+}
+
+
+void SIPEndPoint::InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry)
+{
+  for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    if (handler->GetState() == SIPHandler::Subscribed &&
+        handler->GetTransport() != NULL &&
+        handler->GetTransport()->GetInterface().Find(entry.GetName()) != P_MAX_INDEX)
+      handler->SendRequest(SIPHandler::Refreshing);
+  }
+}
+
+
 #endif // OPAL_SIP
 
 // End of file ////////////////////////////////////////////////////////////////
