@@ -23,14 +23,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20785 $
- * $Author: csoutheren $
- * $Date: 2008-08-31 06:30:59 +0200 (dim 31 aoû 2008) $
+ * $Revision: 21097 $
+ * $Author: dsandras $
+ * $Date: 2008-09-21 11:15:43 +0000 (Sun, 21 Sep 2008) $
  */
 
 #include <ptlib.h>
-
 #include <opal/buildopts.h>
+
 #if OPAL_SIP
 
 #ifdef __GNUC__
@@ -449,12 +449,14 @@ void SIPURL::Sanitise(UsageContext context)
 }
 
 
-#if P_DNS
+#if OPAL_PTLIB_DNS
 PBoolean SIPURL::AdjustToDNS(PINDEX entry)
 {
   // RFC3263 states we do not do lookup if explicit port mentioned
-  if (GetPortSupplied())
-    return PTrue;
+  if (GetPortSupplied()) {
+    PTRACE(4, "SIP\tNo SRV lookup as has explicit port number.");
+    return true;
+  }
 
   // Or it is a valid IP address, not a domain name
   PIPSocket::Address ip = GetHostName();
@@ -1026,6 +1028,7 @@ void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
   info.name = str.Left(endFirstToken);
   info.version = str(endFirstToken+1, endSecondToken);
   info.vendor = GetOrganization();
+  info.comments = str.Mid(endSecondToken+1).Trim();
 }
 
 
@@ -1033,21 +1036,31 @@ void SIPMIMEInfo::SetProductInfo(const PString & ua, const OpalProductInfo & inf
 {
   PString userAgent = ua;
   if (userAgent.IsEmpty()) {
+    PString comments;
+
     PINDEX pos;
     PCaselessString temp = info.name;
-    temp.Replace(' ', '-', PTrue);
-    while ((pos = temp.FindSpan(UserAgentTokenChars)) != P_MAX_INDEX)
-      temp.Delete(pos, 1);
+    if ((pos = temp.FindSpan(UserAgentTokenChars)) != P_MAX_INDEX) {
+      comments += temp.Mid(pos);
+      temp.Delete(pos, P_MAX_INDEX);
+    }
+
     if (!temp.IsEmpty()) {
       userAgent = temp;
 
       temp = info.version;
-      temp.Replace(' ', '-', PTrue);
       while ((pos = temp.FindSpan(UserAgentTokenChars)) != P_MAX_INDEX)
         temp.Delete(pos, 1);
       if (!temp.IsEmpty())
         userAgent += '/' + temp;
     }
+
+    if (info.comments.IsEmpty() || info.comments[0] == '(')
+      comments += info.comments;
+    else
+      comments += '(' + info.comments + ')';
+
+    userAgent &= comments;
   }
 
   if (!userAgent.IsEmpty())
@@ -1527,7 +1540,6 @@ SIP_PDU::SIP_PDU(const SIP_PDU & request,
   mime.SetCSeq(requestMIME.GetCSeq());
   mime.SetVia(requestMIME.GetVia());
   mime.SetRecordRoute(requestMIME.GetRecordRoute());
-  SetAllow();
 
   /* Use extra parameter as redirection URL in case of 302 */
   if (code == SIP_PDU::Redirection_MovedTemporarily) {
@@ -1549,7 +1561,8 @@ SIP_PDU::SIP_PDU(const SIP_PDU & request,
 
 
 SIP_PDU::SIP_PDU(const SIP_PDU & pdu)
-  : method(pdu.method)
+  : PSafeObject(pdu)
+  , method(pdu.method)
   , statusCode(pdu.statusCode)
   , uri(pdu.uri)
   , versionMajor(pdu.versionMajor)
@@ -1635,8 +1648,6 @@ void SIP_PDU::Construct(Methods meth,
   str << ";branch=z9hG4bK" << branch << ";rport";
 
   mime.SetVia(str);
-
-  SetAllow();
 }
 
 
@@ -1686,19 +1697,18 @@ PBoolean SIP_PDU::SetRoute(const PStringList & set)
 }
 
 
-void SIP_PDU::SetAllow(void)
+void SIP_PDU::SetAllow(unsigned bitmask)
 {
   PStringStream str;
-  PStringList methods;
   
-  for (PINDEX i = 0 ; i < SIP_PDU::NumMethods ; i++) {
-    PString method(MethodNames[i]);
-    if (method.Find("SUBSCRIBE") == P_MAX_INDEX && method.Find("REGISTER") == P_MAX_INDEX)
-      methods += method;
+  for (Methods method = Method_INVITE ; method < SIP_PDU::NumMethods ; method = (Methods)(method+1)) {
+    if ((bitmask&(1<<method))) {
+      if (!str.IsEmpty())
+        str << ',';
+      str << method;
+    }
   }
   
-  str << setfill(',') << methods << setfill(' ');
-
   mime.SetAllow(str);
 }
 
@@ -1804,16 +1814,18 @@ bool SIP_PDU::SendResponse(OpalTransport & transport, SIP_PDU & response)
   else {
     // get Via from From field
     PString from = mime.GetFrom();
-    PINDEX j = from.Find (';');
-    if (j != P_MAX_INDEX)
-      from = from.Left(j); // Remove all parameters
-    j = from.Find ('<');
-    if (j != P_MAX_INDEX && from.Find ('>') == P_MAX_INDEX)
-      from += '>';
+    if (!from.IsEmpty()) {
+      PINDEX j = from.Find (';');
+      if (j != P_MAX_INDEX)
+        from = from.Left(j); // Remove all parameters
+      j = from.Find ('<');
+      if (j != P_MAX_INDEX && from.Find ('>') == P_MAX_INDEX)
+        from += '>';
 
-    SIPURL url(from);
+      SIPURL url(from);
 
-    newAddress = OpalTransportAddress(url.GetHostName()+ ":" + PString(PString::Unsigned, url.GetPort()), defaultPort, "udp$");
+      newAddress = OpalTransportAddress(url.GetHostName()+ ":" + PString(PString::Unsigned, url.GetPort()), defaultPort, "udp$");
+    }
   }
 
   return response.Write(transport, newAddress);
@@ -1834,7 +1846,13 @@ void SIP_PDU::PrintOn(ostream & strm) const
 
 PBoolean SIP_PDU::Read(OpalTransport & transport)
 {
+  if (!transport.IsOpen()) {
+    PTRACE(1, "SIP\tAttempt to read PDU from closed tansport " << transport);
+    return PFalse;
+  }
+
   PStringStream datagram;
+  PBYTEArray pdu;
 
   istream * stream;
   if (transport.IsReliable())
@@ -1842,28 +1860,22 @@ PBoolean SIP_PDU::Read(OpalTransport & transport)
   else {
     stream = &datagram;
 
-    PBYTEArray pdu;
-    if (transport.ReadPDU(pdu))
-      datagram = PString((char *)pdu.GetPointer(), pdu.GetSize());
-  }
+    if (!transport.ReadPDU(pdu))
+      return false;
 
-  if (!transport.IsOpen()) {
-    PTRACE(1, "SIP\tAttempt to read PDU from closed tansport " << transport);
-    return PFalse;
+    datagram = PString((char *)pdu.GetPointer(), pdu.GetSize());
   }
 
   // get the message from transport/datagram into cmd and parse MIME
   PString cmd;
-  *stream >> cmd >> mime;
+  *stream >> cmd;
 
-  if (!stream->good()) {
-    PTRACE_IF(1, transport.GetErrorCode(PChannel::LastReadError) != PChannel::NoError,
-              "SIP\tPDU Read failed: " << transport.GetErrorText(PChannel::LastReadError));
-    return PFalse;
-  }
-
-  if (cmd.IsEmpty()) {
-    PTRACE(2, "SIP\tNo Request-Line or Status-Line received on " << transport);
+  if (!stream->good() || cmd.IsEmpty()) {
+    if (stream == &datagram) {
+      transport.setstate(ios::failbit);
+      PTRACE(1, "SIP\tInvalid datagram from " << transport.GetLastReceivedAddress()
+                << " - " << pdu.GetSize() << " bytes.\n" << hex << setprecision(2) << pdu << dec);
+    }
     return PFalse;
   }
 
@@ -1871,7 +1883,7 @@ PBoolean SIP_PDU::Read(OpalTransport & transport)
     // parse Response version, code & reason (ie: "SIP/2.0 200 OK")
     PINDEX space = cmd.Find(' ');
     if (space == P_MAX_INDEX) {
-      PTRACE(2, "SIP\tBad Status-Line received on " << transport);
+      PTRACE(2, "SIP\tBad Status-Line \"" << cmd << "\" received on " << transport);
       return PFalse;
     }
 
@@ -1885,7 +1897,7 @@ PBoolean SIP_PDU::Read(OpalTransport & transport)
     // parse the method, URI and version
     PStringArray cmds = cmd.Tokenise( ' ', PFalse);
     if (cmds.GetSize() < 3) {
-      PTRACE(2, "SIP\tBad Request-Line received on " << transport);
+      PTRACE(2, "SIP\tBad Request-Line \"" << cmd << "\" received on " << transport);
       return PFalse;
     }
 
@@ -1906,10 +1918,19 @@ PBoolean SIP_PDU::Read(OpalTransport & transport)
   }
 
   if (versionMajor < 2) {
-    PTRACE(2, "SIP\tInvalid version received on " << transport);
+    PTRACE(2, "SIP\tInvalid version (" << versionMajor << ") received on " << transport);
     return PFalse;
   }
 
+  // Getthe MIME fields
+  *stream >> mime;
+  if (!stream->good() || mime.IsEmpty()) {
+    PTRACE(2, "SIP\tInvalid MIME received on " << transport);
+    transport.clear(); // Clear flags so BadRequest response is sent by caller
+    return PFalse;
+  }
+
+
   // get the SDP content body
   // if a content length is specified, read that length
   // if no content length is specified (which is not the same as zero length)
@@ -2188,6 +2209,7 @@ PBoolean SIPTransaction::Start()
   else
     completionTimer = endpoint.GetNonInviteTimeout();
 
+  PTRACE(4, "SIP\tTransaction timers set: retry=" << retryTimer << ", completion=" << completionTimer);
   return true;
 }
 
@@ -2442,29 +2464,16 @@ void SIPTransaction::SetTerminated(States newState)
 
 ////////////////////////////////////////////////////////////////////////////////////
 
-SIPInvite::SIPInvite(SIPConnection & connection, OpalTransport & transport)
+SIPInvite::SIPInvite(SIPConnection & connection, OpalTransport & transport, OpalRTPSessionManager * sm)
   : SIPTransaction(connection, transport, Method_INVITE)
 {
   mime.SetDate() ;                             // now
+  SetAllow(connection.GetEndPoint().GetAllowedMethods());
   mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
 
-  m_SDP = new SDPSessionDescription();
-  if (!connection.OnSendSDP(false, rtpSessions, *m_SDP)) {
-    delete m_SDP;
-    m_SDP = NULL;
-  }
+  if (sm != NULL)
+    rtpSessions.CopyFromMaster(*sm);
 
-  connection.OnCreatingINVITE(*this);
-}
-
-
-SIPInvite::SIPInvite(SIPConnection & connection, OpalTransport & transport, OpalRTPSessionManager & sm)
-  : SIPTransaction(connection, transport, Method_INVITE)
-{
-  mime.SetDate() ;                             // now
-  mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
-
-  rtpSessions.CopyFromMaster(sm);
   m_SDP = new SDPSessionDescription();
   if (!connection.OnSendSDP(false, rtpSessions, *m_SDP)) {
     delete m_SDP;
@@ -2550,6 +2559,7 @@ SIPRegister::SIPRegister(SIPEndPoint & ep,
   mime.SetExpires(params.m_expire);
 
   SetRoute(routeSet);
+  SetAllow(ep.GetAllowedMethods());
 }
 
 
@@ -2577,6 +2587,17 @@ SIPSubscribe::Params::Params(PredefinedPackages pkg)
 }
 
 
+SIPSubscribe::PredefinedPackages SIPSubscribe::GetEventPackage(const PString & name)
+{
+  if (name *= "message-summary")
+    return SIPSubscribe::MessageSummary;
+  else if (name *= "presence")
+    return SIPSubscribe::Presence;
+ 
+  return SIPSubscribe::NumPredefinedPackages;
+}
+
+
 PString SIPSubscribe::GetEventPackageName(PredefinedPackages pkg)
 {
   static const char * const names[NumPredefinedPackages] = {
@@ -2620,6 +2641,7 @@ SIPSubscribe::SIPSubscribe(SIPEndPoint & ep,
   mime.SetExpires(params.m_expire);
 
   SetRoute(routeSet);
+  SetAllow(ep.GetAllowedMethods());
 }
 
 
@@ -2627,7 +2649,7 @@ SIPSubscribe::SIPSubscribe(SIPEndPoint & ep,
 
 SIPPublish::SIPPublish(SIPEndPoint & ep,
                        OpalTransport & trans,
-                       const PStringList & /*routeSet*/,
+                       const PStringList & routeSet,
                        const SIPURL & targetAddress,
                        const PString & id,
                        const PString & sipIfMatch,
@@ -2658,6 +2680,8 @@ SIPPublish::SIPPublish(SIPEndPoint & ep,
   mime.SetEvent("presence");
   mime.SetContentType("application/pidf+xml");
 
+  SetRoute(routeSet);
+
   if (!body.IsEmpty())
     entityBody = body;
 }
@@ -2678,12 +2702,16 @@ SIPRefer::SIPRefer(SIPConnection & connection, OpalTransport & transport, const
 }
 
 
-void SIPRefer::Construct(SIPConnection & connection, OpalTransport & /*transport*/, const SIPURL & refer, const SIPURL & referred_by)
+void SIPRefer::Construct(SIPConnection & connection, OpalTransport & /*transport*/, const SIPURL & refer, const SIPURL & _referred_by)
 {
+  SIPURL referred_by = _referred_by;
+
   mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
   mime.SetReferTo(refer.AsQuotedString());
-  if(!referred_by.IsEmpty())
+  if(!referred_by.IsEmpty()) {
+    referred_by.SetDisplayName(PString::Empty());
     mime.SetReferredBy(referred_by.AsQuotedString());
+  }
 }
 
 
@@ -2724,7 +2752,7 @@ SIPMessage::SIPMessage(SIPEndPoint & ep,
   SIPURL myAddress("\"" + displayName + "\" <" + partyName + ">"); 
 
   SIP_PDU::Construct(Method_MESSAGE,
-                     "sip:"+address.GetUserName()+"@"+address.GetHostName(),
+                     address.AsQuotedString(),
                      address.AsQuotedString(),
                      myAddress.AsQuotedString()+";tag="+OpalGloballyUniqueID().AsString(),
                      id,
@@ -2754,7 +2782,7 @@ SIPPing::SIPPing(SIPEndPoint & ep,
   SIPURL myAddress("\"" + displayName + "\" <" + partyName + ">"); 
   
   SIP_PDU::Construct(Method_PING,
-                     "sip:"+address.GetUserName()+"@"+address.GetHostName(),
+                     address.AsQuotedString(),
                      address.AsQuotedString(),
                      // myAddress.AsQuotedString()+";tag="+OpalGloballyUniqueID().AsString(),
                      "sip:"+address.GetUserName()+"@"+address.GetHostName(),
@@ -2828,6 +2856,8 @@ SIPOptions::SIPOptions(SIPEndPoint & ep,
                      endpoint.GetNextCSeq(),
                      viaAddress);
   mime.SetAccept("application/sdp, application/media_control+xml, application/dtmf, application/dtmf-relay");
+
+  SetAllow(ep.GetAllowedMethods());
 }
 
 
